/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_ABSTRACT_INTERMEDIATE_TREE_NODE__
#define __DD_ABSTRACT_INTERMEDIATE_TREE_NODE__

#include "DDTreeNode.h"

#ifndef __MIN_TREE_CHILD_NODES__
#define __MIN_TREE_CHILD_NODES__ 3
#endif

namespace addibit {

    class AbstractIntermediateTreeNode : public TreeNode {
    
        private:
        const TreeNode * mChildBase[ __MIN_TREE_CHILD_NODES__ ];
		std::vector<TreeNode const * > mChild;               
		
		const uint_fast16_t mMaxChildrenCount;		
    
        protected:
        uint_fast16_t mChildrenCount; 

        public:
        AbstractIntermediateTreeNode ( const uint_fast16_t children ) :    
            mMaxChildrenCount( children ),
            mChildrenCount( 0 ) {}
            
		virtual ~AbstractIntermediateTreeNode () {
            for ( uint_fast16_t index = 0; index < mChildrenCount; index++ ) {
                delete child( index );
            } 
        }                
        
        inline const TreeNode * child ( const uint_fast16_t index ) const {
            if ( index < __MIN_TREE_CHILD_NODES__ ) {
                return mChildBase[ index ];
            } else {
                return mChild[ index - __MIN_TREE_CHILD_NODES__ ];
            }
        }     
        
        inline const TreeNode * last_child () const {
            return child( mChildrenCount - 1 );
        }
        
        void insert ( const TreeNode * const );
        void insert ( const uint_fast16_t, const TreeNode * const );    
        
        inline uint_fast16_t getChildrenCount () const {
            return mChildrenCount;
        }       

        inline uint_fast16_t getMaxChildrenCount () const {
            return mMaxChildrenCount;
        }  
        
        bool isLeaf () const {
            return false;
        }
        
        uint_fast64_t getRaysCount () const;
        uint_fast32_t getSubNodesCount () const;
        
		virtual void narrow ( EdgeCandidatesContainer & candidates ) const { 

            for ( int_fast16_t index = mChildrenCount - 1; index >= 0; index-- ) {
                child( index )->narrow( candidates );
            } 
        }        
        
        virtual bool verify ( EdgeCandidate & candidate ) const {
            bool redundant = false;
              
            for ( int_fast16_t index = mChildrenCount - 1; !redundant && index >= 0; index-- ) {
                redundant |= child( index )->verify( candidate );
            }

            return redundant;
        }            

        bool isEmpty () const {
            return false;
        }            
        
        const std::string toGraphVizString ( const uint_fast16_t ) const;  
    };
}

#endif /* __DD_ABSTRACT_INTERMEDIATE_TREE_NODE__ */
