/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_ABSTRACT_LEAF_NODE__
#define __DD_ABSTRACT_LEAF_NODE__

#include "DDBitListArray.h"
#include "DDTreeNode.h"

#include "DDRayRefContainerToBitListArrayTransformer.h"

namespace addibit {

    class AbstractLeafNode : virtual public TreeNode {
        
        private:      
        char * const __memory;
        
        inline void narrow_ref ( EdgeCandidatesContainer & container ) const {      
                                 
            const __BIT_LIST_RAW_DATA_TYPE__ * __entry = mArray.data_begin();
                    
            while ( mArray.next_feasible( container.candidate(), __entry ) ) {
                container.push( __entry, mRefId[ ( __entry - mArray.data_begin() ) / __DD_BLOCK_COUNT__ ] );
                __entry += __DD_BLOCK_COUNT__;
            }
        }                                   

        protected:
        BitListArray mArray; 
        uintptr_t * mRefId;   
        
        public:
		AbstractLeafNode ( const RayReferenceContainer & candidates ) :
            __memory( reinterpret_cast<char *>( new __BIT_LIST_RAW_DATA_TYPE__[ candidates.size() * __DD_BLOCK_COUNT__ ] ) ),
            mArray( candidates.size(), __memory ),
            mRefId( new uintptr_t[ candidates.size() ] ) {
                
            DD_LOG_DEBUG( "Creating a new leaf node with " << candidates.size() << " entries." )
                
            RayRefContainerToBitListArrayTransformer transformer( mArray, mRefId );
            candidates.iterate( transformer );
        }

        virtual ~AbstractLeafNode () {
            delete[] __memory;
            delete[] mRefId;
        }    
        
        void narrow ( EdgeCandidatesContainer & container ) const {
            
            DD_LOG_DEBUG( "A leaf node with " << mArray.entries() << " entries has been reached" )
		    narrow_ref( container );
		}
        
        bool verify ( EdgeCandidate & candidate ) const {   
            
            for ( int_fast32_t hit_index = mArray.contains( candidate.data() ); hit_index < mArray.entries(); hit_index = mArray.contains( candidate.data(), hit_index + 1 ) ) {
                if ( likely ( mRefId[ hit_index ] != candidate.illegal_1() ) && likely( mRefId[ hit_index ] != candidate.illegal_2() ) ) {
                    return true;
                }
            }
            
            return false;
        }
        
        uint_fast32_t getSubNodesCount () const {
            return 0;
        }
        
        bool isLeaf () const {
            return true;
        }        

        uint_fast64_t getRaysCount () const {
            return mArray.entries();
        }

        uint_fast32_t getLeafNodesCount () const {
            return isEmpty() ? 0 : 1;
        }

        bool isEmpty () const {
            return mArray.empty();
        } 
        
        virtual const std::string toGraphVizString ( const uint_fast16_t ) const;                    
    };
}

#endif /* __DD_ABSTRACT_LEAF_NODE__ */
