/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_ABSTRACT_TREE_CREATOR__
#define __DD_ABSTRACT_TREE_CREATOR__

#include "DDTreeNode.h"
#include "DDConditionedRayReferenceContainerList.h"

namespace addibit {

    enum TreeTargetPhase { NARROWING, VERIFICATION };

    class AbstractTreeCreator {
    
        private:
        TreeTargetPhase mTarget;
    
        public:      
        AbstractTreeCreator () :
            mTarget( NARROWING ) {}

        virtual ~AbstractTreeCreator () {}
            
        virtual const TreeNode * createHDTree ( RayReferenceContainer && ) = 0;
        virtual const TreeNode * createLDTree ( RayReferenceContainer && ) = 0;
        virtual const ConditionedRayReferenceContainerList * createList ( RayReferenceContainer && ) = 0;
        
        void setTarget ( const TreeTargetPhase target ) {
            mTarget = target;
        }
        
        bool build_for_narrowing () const {
            return mTarget == NARROWING;
        }
        
        virtual uint_fast16_t getNarrowingCandidateSize () const {
            return sizeof( NarrowingCandidate ) + sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) * ( __DD_BLOCK_COUNT__ - 1 );
        }
        
        virtual uint_fast16_t getAdjacencyCandidateSize () const {
            return sizeof( AdjacencyCandidate ) + sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) * ( __DD_BLOCK_COUNT__ - 1 );
        }
        
        virtual uint_fast16_t getAdjacencyStackSize () const {
            return 0;
        }
        
        virtual uint_fast16_t getAdjacencyCandidateOffset () const {
            return 0;
        }
    };
}

#endif /* __DD_ABSTRACT_TREE_CREATOR__ */
