/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_BINARY_DYNAMIC_VANTAGE_POINT_STATIC_DISTANCE_VISITOR__
#define __DD_BINARY_DYNAMIC_VANTAGE_POINT_STATIC_DISTANCE_VISITOR__

#include "DDBinaryVantagePointVisitor.h"

namespace addibit {

    class BinaryDynamicVantagePointStaticDistanceVisitor : public BinaryVantagePointVisitor {

        private:
        const double mCommonFactor;
        double mCurrentDistance;

        protected:
        void onInit ( const RayReference & ) {
            /* No init phase for static distance. */
        }

        bool subset_condition ( const RayReference & ref ) const {
            return mVantagePoint.getCommonPopCnt( ref.activeSet ) < mCurrentDistance;
        }

        public:
        BinaryDynamicVantagePointStaticDistanceVisitor ( const unsigned int depth,
                                                         const unsigned int capacity,
                                                         const double factor ) :
            BinaryVantagePointVisitor( depth, capacity ),
            mCommonFactor( factor ) {}

        void before ( const RayReferenceContainer & container ) {
            BinaryVantagePointVisitor::before( container );
            mVantagePoint = container.at( current_depth() ).activeSet; /* Not the best solution! */
            mCurrentDistance = mCommonFactor * mVantagePoint.getPopulationCount();
        }
    };
}

#endif // __DD__BINARY_DYNAMIC_VANTAGE_POINT_STATIC_DISTANCE_VISITOR__
