/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BINARY_STATIC_VANTAGE_POINT_VISITOR__
#define __DD_BINARY_STATIC_VANTAGE_POINT_VISITOR__

#include <deque>

#include "DDBinaryVantagePointVisitor.h"

namespace addibit {

    class BinaryStaticVantagePointVisitor : public BinaryVantagePointVisitor {

        private:
        const std::vector<BitList> mPoints;
        unsigned int mCurrent;

        protected:
        bool can_proceed ( const unsigned int index ) const {
            return BinaryVantagePointVisitor::can_proceed( index ) && mCurrent < mPoints.size();
        }

        bool can_proceed ( const RayReferenceContainer & container ) const {
            return container.pop_cnt_range() > 0;
        }

        bool use_vp_for_verification () const {
            return false;
        }

        public:
        BinaryStaticVantagePointVisitor ( const unsigned int depth,
                                          const unsigned int capacity,
                                          const std::vector<BitList> & points ) :
            BinaryVantagePointVisitor( depth, capacity ),
            mPoints( points ),
            mCurrent( 0 ) {}

        void before ( const RayReferenceContainer & container ) {
            BinaryVantagePointVisitor::before( container );

            mVantagePoint = mPoints[ mCurrent++ ];
        }

        void after () {
            BinaryVantagePointVisitor::after();
            mCurrent--;
        }              
    };
}

#endif // __DD_BINARY_STATIC_VANTAGE_POINT_VISITOR__
