/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_BIT_COUNT_TABLE__
#define __DD_BIT_COUNT_TABLE__

#include <limits.h>
#include <stdint.h>

namespace addibit {

	class BitCountTable {

		private:
		unsigned char data[ USHRT_MAX ];

		unsigned char getBitCountInMap ( unsigned long map ) const;

		public:
		BitCountTable ();
		~BitCountTable () {}

		inline uint_fast16_t lookUp ( const unsigned long & map ) const {
#ifdef __SSE4_2__	
			return static_cast<uint_fast16_t>( __builtin_popcountl ( map ) );
#else
    #if defined _LP64 || defined __LP64__
				return data[ map >> 48 ] + data[ ( map >> 32 ) & 0x000000000000FFFF ] + data[ ( map >> 16 ) & 0x000000000000FFFF ] + data[map & 0x000000000000FFFF];
    #else
				return data[ map >> 16 ] + data[map & 0x000000000000FFFF];
    #endif // _LP64 || __LP64__
#endif // __SSE4_2__
		}

		inline uint_fast16_t lookUp ( const unsigned long * const first, const unsigned long * const second, const unsigned long blocks ) const {

            uint_fast16_t count;

	#ifdef __SSE4_2__	
			count = __builtin_popcountl ( first[0] & second[0] );
	#else
			count = lookUp ( first[0] & second[0] );
	#endif // __SSE4_2__

			for ( uint_fast16_t block = 1; block < blocks; block++ ) {
#ifdef __SSE4_2__	
				count += __builtin_popcountl ( first[block] & second[block] );
#else
				count += lookUp ( first[block] & second[block] );
#endif // __SSE4_2__
			}

			return count;
		}

		inline uint_fast16_t lookUp ( const unsigned long * const map, const unsigned long blocks ) const {
            uint_fast16_t count;

	#ifdef __SSE4_2__	
			count = __builtin_popcountl ( map[0] );
	#else
			count = lookUp ( map[0] );
	#endif // __SSE4_2__

			for ( uint_fast16_t block = 1; block < blocks; block++ ) {
#ifdef __SSE4_2__	
				count += __builtin_popcountl ( map[block] );
#else
				count += lookUp ( map[block] );
#endif // __SSE4_2__
			}

			return count;
		}
	};
}

#endif // __DD_BIT_COUNT_TABLE__
