/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDBitList.h"

#include <sstream>
#include <iostream>

using namespace addibit;

const BitCountTable BitList::BIT_COUNT_TABLE;
uint_fast16_t BitList::LAST_POP_CNT_RESULT = 0;

void BitList::init ( const unsigned int blocks, const unsigned long value ) {

#ifdef __DOUBLE_BLOCK_OPT__
	mFirstBlock = value;
	mSecondBlock = value;

	__data = new unsigned long[blocks - 2];

	for ( unsigned int block = 0; block < blocks-2; block++ ) {
		__data[block] = value;
	}

#else
	#ifdef __SINGLE_BLOCK_OPT__

	mFirstBlock = value;

	__data = new unsigned long[blocks - 1];

	for ( unsigned int block = 0; block < blocks-1; block++ ) {
		__data[block] = value;
	}

	#else

	delete __data;
	__data = new unsigned long[blocks];

	for ( unsigned int block = 0; block < blocks; block++ ) {
		__data[block] = value;
	}

	#endif // __SINGLE_BLOCK_OPT__
#endif //__DOUBLE_BLOCK_OPT__
}

void BitList::resize ( const unsigned long blocks ) {

	unsigned long* copy = NULL;
    
	copy = new unsigned long[ blocks ];

    if ( likely( blocks > __DD_BLOCK_COUNT__ ) ) {
        memset ( copy, 0, blocks * sizeof( unsigned long ) );
        memcpy ( copy, __data, __DD_BLOCK_COUNT__ * sizeof( unsigned long ) );
    } else {
        memcpy ( copy, __data, blocks * sizeof( unsigned long ) );
    }

	delete[] __data;
	__data = copy;
}

const std::string BitList::toString () const {
	return toString( getCapacity() );
}

const std::string BitList::toString ( const unsigned int length ) const {
	std::stringstream output;	

	for ( uint_fast16_t pos = 0; pos < length; pos++ ) {
	    if ( isSet( pos ) ) {
		    output << "1";
		} else {
			output << "0";
		}
	}

	return output.str();
}
