/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_BIT_SELECT_STRATEGY__
#define __DD_BIT_SELECT_STRATEGY__

#include <vector>
#include <deque>

#include "DDRayReference.h"
#include "DDBitCountVisitor.h"

namespace addibit {

    class BitSelectStrategy {

        protected:
        const uint_fast16_t mPlane;  
        const double_t * __probability;
        
        public:
        BitSelectStrategy ( const uint_fast16_t plane ) :
            mPlane( plane ),
            __probability( NULL ) {}         
        
        BitSelectStrategy ( const std::vector<unsigned long> & stats,
                            const uint64_t elements,
                            const uint_fast16_t plane ) :
            mPlane( plane ),
            __probability( calc_probability( stats, elements ) ) {}     

        virtual ~BitSelectStrategy () {
            if ( __probability ) {
                delete[] __probability;
            }
        }

        virtual std::vector<unsigned int> createMapping () const = 0;

        uint_fast16_t getBitCount () const {
            return mPlane;
        }

        const double_t * const & probability () const {
            return __probability;
        }
        
        double_t * getFragHitProbability ( const BitList * const frag, const uint_fast16_t fcount ) const {
            double_t * result = new double_t[ fcount ];
            
            for ( uint_fast16_t f_index = 0; f_index < fcount; ++f_index ) {
                result[ f_index ] = 0;
                uint_fast16_t i = 0;
                
                do {
                    __BIT_LIST_RAW_DATA_TYPE__ data = frag[f_index][i];
                    
                    while ( data ) {
                        const uint_fast16_t index = i * __BLOCK_SIZE__ + __builtin_ctzl( data );
                        result[ f_index ] += __probability[ index ];
                        __REMOVE_LS_BIT( data );
                    }
                } while ( ++i < __DD_BLOCK_COUNT__ );

            }
            
            return result;
        }

        const double_t * calc_probability ( const std::vector<unsigned long> & stats, const uint64_t elements ) const;
    };
}

#endif // __DD_BIT_SELECT_STRATEGY__
