/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_CACHE_ENTRY_CONTAINER__
#define __DD_CACHE_ENTRY_CONTAINER__

#include <deque>

#include "DDLogging.h"

namespace addibit {

	template <class RAY_TYPE, class CACHE_ENTRY_TYPE>
	class CacheEntryNode;	
	
	template <class RAY_TYPE, class CACHE_ENTRY_TYPE>
	class CacheEntryContainer {
	
	    private:
	    const uint_fast16_t mCapacity;	    
	    CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> ** const mData;
	    
	    public:
	    CacheEntryContainer ( const uint_fast16_t count ) :
	        mCapacity( count ),
	        mData( new CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> *[ count ] ) {}
	    
	    CacheEntryContainer ( const CacheEntryContainer<RAY_TYPE, CACHE_ENTRY_TYPE> & other ) :
	        mCapacity( other.mCapacity ),
	        mData( new CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> *[ mCapacity ] ) {
	        
            for ( uint_fast16_t index = 0; index < mCapacity; index++ ) {
                mData[ index ] = other.mData[ index ];
            }	            
	    }	    
	        
	    ~CacheEntryContainer () {
	        delete[] mData;
	    }
        
        inline uint_fast16_t successors () const {
            return mCapacity;
        }
	    
	    inline CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> ** begin () {
	        return mData;
	    }
	    
	    inline CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> ** end () {
	        return mData + mCapacity;
	    }	
	    
	    inline const CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * const * begin () const {
	        return mData;
	    }
	    
	    inline const CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * const * end () const {
	        return mData + mCapacity;
	    }		    
	    
	    CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * & operator[] ( const uint_fast16_t position ) {
	        return mData[ position ];
	    }
	    
	    const CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * operator[] ( const uint_fast16_t position ) const {
	        return mData[ position ];
	    }	    
	}; 
}

#endif /* __DD_CACHE_ENTRY_CONTAINER__ */
