/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_CACHE_ENTRY_NODE__
#define __DD_CACHE_ENTRY_NODE__

#include "DDCacheEntry.h"

namespace addibit {

    template<class RAY_TYPE, class CACHE_ENTRY_TYPE>
    class CacheEntryNode {
    
        private:
        CACHE_ENTRY_TYPE * mData;
        const uint_fast16_t mId;
        const uint_fast16_t mThreadId;
		    
        public:
        CacheEntryNode ( CACHE_ENTRY_TYPE * data ) :
            mData( data ),
            mId( 0 ),
            mThreadId( 0 ) {}
            
        CacheEntryNode ( CACHE_ENTRY_TYPE * data,
                        const uint_fast16_t id ) :
            mData( data ),
            mId( id ),
            mThreadId( 0 ) {}    
            
        CacheEntryNode ( CACHE_ENTRY_TYPE * data,
                        const uint_fast16_t id,
                        const uint16_t thread_id ) :
            mData( data ),
            mId( id ),
            mThreadId( thread_id ) {}                      
            
        virtual ~CacheEntryNode () {
            delete mData;
        }
        
		virtual bool push ( std::shared_ptr<const RAY_TYPE> ) = 0;
		virtual void insert ( CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> *, const uint_fast16_t ) = 0;    
		virtual void enum_successors ( std::deque<CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * > & ) {}
		
		inline CACHE_ENTRY_TYPE & data () {
		    return *mData;
		}		
		
		inline const CACHE_ENTRY_TYPE & data () const {
		    return *mData;
		}
        
        inline uint_fast16_t id () const {
            return mId;
        }
        
        inline uint_fast16_t thread_id () const {
            return mThreadId;
        }
        
        inline void prepare_for_test () {
            mData->prepare_for_test();
        }
        
        virtual const std::string toString () const {
            std::stringstream result;
            
            if ( mData == NULL ) {
                result << "EMPTY";
            } else {
                result << mData->toString() << ", id=" << id() << ", t_id=" << thread_id();
            }            
            
            return result.str();
        }
        
        virtual const std::string toGraphVizString () const {
            std::stringstream result;
            
            result << "p" << static_cast<const void*>( this ) << " [shape = rectangle, label=\""
                   << toString() << "\"];"; 
            
            return result.str();
        }
    };
}

#endif /* __DD_CACHE_ENTRY_NODE__ */
