/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DDCombTestSingleSetRayCacheEntry.h"

using namespace addibit;

CombTestSingleSetRayCacheEntry * CombTestSingleSetRayCacheEntry::inequality_entry ( const uint_fast16_t rows,
                                                                                    const uint_fast16_t plane,
                                                                                    const BitList & pattern ) {
    return new InequalityCombTestSingleSetRayCacheEntry( rows, plane, pattern );                          
}

CombTestSingleSetRayCacheEntry * CombTestSingleSetRayCacheEntry::inequality_entry ( const uint_fast16_t rows,
                                                                                    const uint_fast16_t plane,
                                                                                    const uint_fast16_t plane_coeff_offset,
                                                                                    const BitList & pattern ) {
    return new InequalityCombTestSingleSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern );                          
}

CombTestSingleSetRayCacheEntry * CombTestSingleSetRayCacheEntry::equality_entry ( const uint_fast16_t rows,
                                                                                  const uint_fast16_t plane,
                                                                                  const uint_fast16_t plane_coeff_offset,
                                                                                  const BitList & pattern ) {
    return new EqualityCombTestSingleSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern );                          
}


bool InequalityCombTestSingleSetRayCacheEntry::proceed ( std::shared_ptr<const SingleSetRay> ray ) {

    if ( ray->isOnThePlane( plane() ) ) {
        insert_zero_ver( ray );
        return true;
    } else {
        return insert_narrow( ray );
    }
}

bool EqualityCombTestSingleSetRayCacheEntry::proceed ( std::shared_ptr<const SingleSetRay> ray ) {

    if ( ray->isOnThePlane( plane() ) ) {
        insert_zero_ver( ray );
        return true;
    } else {
        insert_narrow( ray );
        return false;
    }
}

const std::string EqualityCombTestSingleSetRayCacheEntry::toString () const {
    std::stringstream result;
    
    result << "[=] " << CombTestSingleSetRayCacheEntry::toString();
    
    return result.str();
} 

const std::string InequalityCombTestSingleSetRayCacheEntry::toString () const {
    std::stringstream result;
    
    result << "[>] " << CombTestSingleSetRayCacheEntry::toString();
    
    return result.str();
} 
