/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_COMB_TEST_SINGLE_SET_RAY_CACHE_ENTRY__
#define __DD_COMB_TEST_SINGLE_SET_RAY_CACHE_ENTRY__

#include "DDSingleSetRay.h"
#include "DDCacheEntry.h"

namespace addibit {

	class CombTestSingleSetRayCacheEntry : public CacheEntry<SingleSetRay> {
	
	    private:
	    RayWrapperContainer<SingleSetRay> mVerificationEntriesHighDeg;
		RayWrapperContainer<SingleSetRay> mVerificationEntriesLowDeg;
		
		const uint_fast16_t mVerDegThreshold;   	
        
        inline static uint_fast16_t getVerDegThreshold () {
            return std::max<unsigned long>( Options::current().verification_degeneracy_threshold(), 1 ) + GLOBAL_EDGE_DIM;
        }	
        
        protected:
        
        using CacheEntry<SingleSetRay>::ray_dim;
        
        inline void insert_zero_ver ( std::shared_ptr<const SingleSetRay> ray ) {
            insert_zero_ver( ray, mPattern.getCommonPopCnt( ray->getActiveSet() ));         
        }          
        
        inline void insert_zero_ver ( std::shared_ptr<const SingleSetRay> ray,
                                      const uint_fast16_t deg ) {
            
            if ( deg > mVerDegThreshold ) {   
                mVerificationEntriesHighDeg.insert( ray );
            } else if ( deg > ray_dim() ) {
                mVerificationEntriesLowDeg.insert( ray );
            }         
        }   
        
        inline bool insert_narrow ( std::shared_ptr<const SingleSetRay> ray ) {
                                        
            const uint_fast16_t deg = mPattern.getCommonPopCnt( ray->getActiveSet() );
            bool result = false;

            if ( deg > mNarDegThreshold && ray->isInside( plane() ) ) {
                result = true;
                mPositiveEntriesHighDeg.insert( ray );
            } else if ( deg > mNarDegThreshold ) {
                mNegativeEntriesHighDeg.insert( ray );
            } else if ( ray->isInside( plane() ) ) {
                result = true;
                mPositiveEntriesLowDeg.insert( ray );
            } else {
                mNegativeEntriesLowDeg.insert( ray );
            } 
            
            insert_zero_ver( ray, deg );
            
            return result;             
        }                 

		public:
		CombTestSingleSetRayCacheEntry ( const uint_fast16_t rows,
                               			 const uint_fast16_t plane,
                                         const BitList & pattern ) :
			CacheEntry<SingleSetRay>( rows, plane, pattern ),
			mVerDegThreshold( getVerDegThreshold() ) {}
			
		CombTestSingleSetRayCacheEntry ( const uint_fast16_t rows,
                               			 const uint_fast16_t plane,
                               			 const uint_fast16_t plane_coeff_offset,
                                         const BitList & pattern ) :
			CacheEntry<SingleSetRay>( rows, plane, plane_coeff_offset, pattern ),
			mVerDegThreshold( getVerDegThreshold() ) {}			

		virtual ~CombTestSingleSetRayCacheEntry () {}
		
        const RayWrapperContainer<SingleSetRay> & verification_hd () const {
            return mVerificationEntriesHighDeg;
        }
        
        const RayWrapperContainer<SingleSetRay> & verification_ld () const {
            return mVerificationEntriesLowDeg;
        }  		
        
        uint_fast16_t getVerificationCount () const {
            return mVerificationEntriesHighDeg.size() + mVerificationEntriesLowDeg.size();
        } 
        
        uint_fast16_t verification_deg_threshold () const {
            return mVerDegThreshold;
        }
        
		static CombTestSingleSetRayCacheEntry * inequality_entry ( const uint_fast16_t, const uint_fast16_t, const BitList & );
        static CombTestSingleSetRayCacheEntry * inequality_entry ( const uint_fast16_t, const uint_fast16_t, const uint_fast16_t, const BitList & );
		static CombTestSingleSetRayCacheEntry * equality_entry ( const uint_fast16_t, const uint_fast16_t, const uint_fast16_t, const BitList & );        
	};
	
	class InequalityCombTestSingleSetRayCacheEntry : public CombTestSingleSetRayCacheEntry {
	
	    public:
	    InequalityCombTestSingleSetRayCacheEntry ( const uint_fast16_t rows,
                               			           const uint_fast16_t plane,
                                                   const BitList & pattern ) :
            CombTestSingleSetRayCacheEntry( rows, plane, pattern ) {}
            
	    InequalityCombTestSingleSetRayCacheEntry ( const uint_fast16_t rows,
                               			           const uint_fast16_t plane,
                                                   const uint_fast16_t plane_coeff_offset,
                                                   const BitList & pattern ) :
            CombTestSingleSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern ) {}            
            
        ~InequalityCombTestSingleSetRayCacheEntry () {}        
            
        bool proceed ( std::shared_ptr<const SingleSetRay> );    
        const std::string toString () const;        
	};
	
	class EqualityCombTestSingleSetRayCacheEntry : public CombTestSingleSetRayCacheEntry {
	
	    public:
	    EqualityCombTestSingleSetRayCacheEntry ( const uint_fast16_t rows,
                               			         const uint_fast16_t plane,
                               			         const uint_fast16_t plane_coeff_offset,
                                                 const BitList & pattern ) :
            CombTestSingleSetRayCacheEntry( rows, plane, plane_coeff_offset, pattern ) {}
            
        ~EqualityCombTestSingleSetRayCacheEntry () {}    
            
        bool proceed ( std::shared_ptr<const SingleSetRay> );
        const std::string toString () const;            
	};	
}

#endif /* __DD_COMB_TEST_SINGLE_SET_RAY_CACHE_ENTRY__ */
