/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_COMMON_BIT_STRATEGY__
#define __DD_COMMON_BIT_STRATEGY__

#include <vector>

#include "DDBitSelectStrategy.h"

namespace addibit {

    class CommonBitStrategy : public BitSelectStrategy {
        
        public:
        CommonBitStrategy ( const std::vector<unsigned long> & stats,
                            const uint64_t elements,
                            const unsigned int plane ) :
            BitSelectStrategy( stats, elements, plane ) {}        

        std::vector<unsigned int> createMapping () const {

            std::vector<unsigned int> mapping;
            const double_t * const & container_prob = probability();
            const double_t target = getTargetValue();

            for ( uint_fast16_t index = 0; index < mPlane; index++ ) {
                uint_fast16_t position = 0;

                for ( uint_fast16_t prev = 0; prev < index; prev++ ) {
                    if ( isOrderingCorrect( container_prob[ prev ], container_prob[ index ], target ) ) {
                        position++;
                    }
                }

                mapping.insert( mapping.begin() + position, index );
            }

            return mapping;
        }

        bool isOrderingCorrect ( const double_t first, const double_t second, const double_t target ) const {
            const double_t first_diff = first > target ? first - target : target - first;
            const double_t second_diff = second > target ? second - target : target - second;

            return ( first_diff < second_diff && closestToTarget() ) || ( first_diff >= second_diff && !closestToTarget() );
        }

        virtual double_t getTargetValue() const = 0;
        
        virtual bool closestToTarget () const {
            return true;
        } 
    };
}

#endif // __DD_COMMON_BIT_STRATEGY&
