/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_CONTAINER_BUCKET__
#define __DD_CONTAINER_BUCKET__

#include "DDRayReference.h"
#include "DDRayRefModifier.h"

namespace addibit {
    
    class ContainerBucket {
      
        private:
        RayReference * __start_ptr;
        RayReference * __end_ptr;
        
        public:        
        ContainerBucket ( RayReference * content = NULL,
                          const uint_fast32_t size = 0 ) :
            __start_ptr( content ),
            __end_ptr( __start_ptr + size ) {}        
            
        ~ContainerBucket () {
            if ( __start_ptr ) {
                delete[] __start_ptr;
            }
        }
        
        inline void operator= ( ContainerBucket && rvalue ) {
            std::swap( __start_ptr, rvalue.__start_ptr );
            std::swap( __end_ptr, rvalue.__end_ptr );
        }
        
        inline RayReference * start_ptr () {
            return __start_ptr;
        }
        
        inline RayReference * end_ptr () {
            return __end_ptr;
        }
        
        inline uint_fast32_t capacity () const {
            return __end_ptr - __start_ptr;
        }
        
        template<class VISITOR>
        void iterate_init ( VISITOR & visitor, const RayReference * __last_element_ptr ) const {  
            for ( const RayReference * __element_ptr = __start_ptr; __element_ptr != __last_element_ptr; __element_ptr++ ) {
                visitor.onInitVisit( __element_ptr[0] );              
            }
        }

        template<class VISITOR>
        inline void iterate_init ( VISITOR & visitor ) const {  
            iterate_init ( visitor, __end_ptr );
        }

        template<class VISITOR>
        void iterate ( VISITOR & visitor, const RayReference * __last_element_ptr ) const {  
            for ( const RayReference * __element_ptr = __start_ptr; __element_ptr != __last_element_ptr; __element_ptr++ ) {
                visitor.onVisit( __element_ptr[0] );              
            }
        }    
        
        template<class VISITOR>
        inline void iterate ( VISITOR & visitor ) const {  
            iterate( visitor, __end_ptr );
        }             
        
        template<class VISITOR>
        void successive_pop ( VISITOR & visitor, const RayReference * __last_element_ptr ) {  
            for ( RayReference * __element_ptr = __start_ptr; __element_ptr != __last_element_ptr; __element_ptr++ ) {
                visitor.onPop( std::move( __element_ptr[0] ) );              
            }
        }    
        
        template<class VISITOR>
        inline void successive_pop ( VISITOR & visitor ) {  
            successive_pop( visitor, __end_ptr );
        }        
        
        void modify ( const RayRefModifier & modifier, RayReference * __last_element_ptr ) {
            for ( RayReference * __element_ptr = __start_ptr; __element_ptr != __last_element_ptr; __element_ptr++ ) {
                modifier.modify( __element_ptr[0] );              
            }
        }
        
        inline void modify ( const RayRefModifier & modifier ) {
            modify( modifier, __end_ptr );
        }        
    };
}

#endif /* __DD_CONTAINER_BUCKET__ */
