/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_CONTAINER_FRAGMENTATION_MAPPING__
#define __DD_CONTAINER_FRAGMENTATION_MAPPING__

#include <vector>

#include "DDLogging.h"
#include "DDRayRefVisitor.h"
#include "DDFragmentationTransformation.h"

namespace addibit {

    class ContainerFragmentationMapping : public RayRefVisitor {

        private:        
        uint_fast32_t mRefCount;
        const FragmentationTransformation & mTransformation;

        FragmentationData * __ref_mapping;
     //   std::deque<std::vector<uint_fast64_t> > __sum;
        
        std::vector<FragmentationData> mLowerBound;
        std::vector<FragmentationData> mUpperBound;
        
        public:
        ContainerFragmentationMapping ( const FragmentationTransformation & ftransformation ) :
            mRefCount( 0 ),       
            mTransformation( ftransformation ),
            __ref_mapping( NULL ) {}
            
        ~ContainerFragmentationMapping () {
            if ( __ref_mapping ) {
                delete[] __ref_mapping;
            }
        }
        
        void insert_empty () {
            mLowerBound.emplace_back( mTransformation.getFragmentCount(), 127 );
            mUpperBound.emplace_back( mTransformation.getFragmentCount() );
        //    __sum.emplace_back( mTransformation.getFragmentCount(), 0 );
        }
        
        void update ( const uint_fast32_t id, const RayReference & ref ) {             
            mLowerBound[ id ] &= __ref_mapping[ ref.id ];
            mUpperBound[ id ] |= __ref_mapping[ ref.id ];
            
          /*  for ( uint_fast16_t index = 0; index < mTransformation.getFragmentCount(); ++index ) {
                __sum[ id ][ index ] += __ref_mapping[ ref.id ].get_frag_value( index );
            } */
        }
        
        const FragmentationData & ref_data ( const uint_fast32_t id ) const {
            return __ref_mapping[ id ];
        }
        
        const FragmentationData & lbound ( const uint_fast32_t id ) const {                
            return mLowerBound[ id ];
        }
        
        const FragmentationData & ubound ( const uint_fast32_t id ) const {                
            return mUpperBound[ id ];
        }        
        
     /*   uint_fast16_t median ( const uint_fast32_t id, const uint_fast16_t fragment ) const {
            return  __sum[ id ][ fragment ] / mRefCount;
        } */
        
        uint_fast32_t registered () const {
            return mLowerBound.size();
        }
        
        void init ( const uint_fast32_t rcount ) {
            mRefCount = rcount;
            __ref_mapping = new FragmentationData[ rcount ];
        }

        void onVisit ( const RayReference & ref ) {
             __ref_mapping[ ref.id ] = mTransformation.convert( ref.activeSet );
             update( 0, ref );
        }
        
        uint_fast16_t getNextOpenFragment ( const uint_fast32_t id ) const;
    };
}

#endif /* __DD_CONTAINER_FRAGMENTATION_MAPPING__ */
