/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_DEGENERACY_CALC_VISITOR__
#define __DD_DEGENERACY_CALC_VISITOR__

#include "DDRayWrapperVisitor.h"
#include "DDConstants.h"

namespace addibit {

    template<class RAY_TYPE>
    class DegeneracyCalcVisitor : public RayWrapperVisitor<RAY_TYPE> {

        private:
        const BitList & mPattern;
    
        uint_fast32_t __run_elements;
        uint_fast32_t __deg_sum;

        public:
        DegeneracyCalcVisitor( const BitList & pattern ) :
            mPattern( pattern ),
            __run_elements( 0 ),
            __deg_sum( 0 ) {}

        inline void onStart ( const uint_fast32_t size ) {
            __run_elements += size;
        }

        inline void process_ray ( const RAY_TYPE & ray ) { 
            __deg_sum += mPattern.getCommonPopCnt( ray.getActiveSet() ) - GLOBAL_RAY_MIN_FACETS;
        }
        
        inline void reset () {
            __run_elements = 0;   
            __deg_sum = 0;            
        }

        inline double_t degeneracy () const {
            return ((double_t)__deg_sum ) / __run_elements;
        }
    };
}

#endif // __DD_DEGENERACY_CALC_VISITOR__
