/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_DIFFERENTIAL_BIT_STRATEGY__
#define __DD_DIFFERENTIAL_BIT_STRATEGY__

#include <vector>

#include "DDBitSelectStrategy.h"

namespace addibit {

    class DifferentialBitStrategy : public BitSelectStrategy {

        protected:
        const double_t * __tree_probability;
        
        virtual double_t getProbabilityCoeff ( const uint_fast16_t index,
                                               const double_t list_prob,
                                               const double_t tree_prob ) const = 0;

        public:
        DifferentialBitStrategy ( const std::vector<unsigned long> & list_container_stats,
                                  const uint64_t list_elements,
                                  const std::vector<unsigned long> & tree_container_stats,
                                  const uint64_t tree_elements,
                                  const uint_fast16_t plane ) :
            BitSelectStrategy( list_container_stats, list_elements, plane ),
            __tree_probability( calc_probability( tree_container_stats, tree_elements ) ) {}     

        virtual ~DifferentialBitStrategy () {
            delete[] __tree_probability;
        }

        std::vector<unsigned int> createMapping () const;
        
        const std::string toString () const;
    };
}

#endif // __DD_DIFFERENTIAL_BIT_STRATEGY__
