
#include "DDFaceProver.h"

using namespace addibit;

FaceProver::FaceProver ( const unsigned int planes, const unsigned int rank ) :
	mPlanes( planes ),
	mTargetRank( rank ),
	mClearance( 0 ),
	mCurrentRank( 0 ) {

	mWorkingSet = new BitRegister[ planes ];

	mCoeffCache = new mpz_t*[ planes ];

    mpz_init( gcd );
    mpz_init( ci_multiplicator );
    mpz_init( si_multiplicator );

	for ( uint_fast16_t r_index = 0; r_index < planes; r_index++ ) {
		mCoeffCache[ r_index ] = new mpz_t[ planes ];

        for ( uint_fast16_t c_index = 0; c_index < planes; c_index++ ) {
            mpz_init( mCoeffCache[ r_index ][ c_index ] );
        }
	}
}

FaceProver::FaceProver ( const FaceProver & other ) :
	mPlanes( other.mPlanes ),
	mTargetRank( other.mTargetRank ),
	mClearance( 0 ),
	mCurrentRank( 0 ) {

	mWorkingSet = new BitRegister[ mPlanes ];

	mCoeffCache = new mpz_t*[ mPlanes ];

	for ( uint_fast16_t r_index = 0; r_index < mPlanes; r_index++ ) {
		mCoeffCache[ r_index ] = new mpz_t[ mPlanes ];

        for ( uint_fast16_t c_index = 0; c_index < mPlanes; c_index++ ) {
            mpz_init( mCoeffCache[ r_index ][ c_index ] );
        }
	}
}

FaceProver::~FaceProver () {
	delete[] mWorkingSet;

	for ( uint_fast16_t plane = 0; plane < mPlanes; plane++ ) {
		delete[] mCoeffCache[plane];
	}

    delete[] mCoeffCache;
}
