/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_FRAGMENTATION_CONDITION__
#define __DD_FRAGMENTATION_CONDITION__

#include "DDFragmentationData.h"
#include "DDTreeNodeCondition.h"

namespace addibit {

    union NarrowingDataResult {
        __m128i __sum;
        int64_t __vec[2];
    };
    
    union VerificationDataResult {
        __m128i __vec;
        long long __cmp_value;
    };

	class FragmentationCondition {

        private:
        FragmentationData __frag_data;

		public:
        FragmentationCondition () {}     
        
		FragmentationCondition ( FragmentationData && frag_data ) :
			__frag_data( std::move( frag_data ) ) {}        
            
		FragmentationCondition ( const FragmentationData & frag_data ) :
			__frag_data( frag_data ) {}
        
        FragmentationCondition ( FragmentationCondition && rvalue ) :
            __frag_data( std::move( rvalue.__frag_data ) ) {}
        
        inline void operator= ( FragmentationCondition && rvalue ) {
            __frag_data = std::move( rvalue.__frag_data );
        }
        
        void write_data ( __BIT_LIST_RAW_DATA_TYPE__ * data ) const {
            FragmentationSegment * __node_data = reinterpret_cast<FragmentationSegment *>( data );
            uint_fast16_t i = 0;
            
            do {
                __node_data[i] = __frag_data[i];
            } while ( ++i < __frag_data.segment_count() );     
        }
        
        inline void write_narrowing_data ( __BIT_LIST_RAW_DATA_TYPE__ * data ) const {
            write_data( data );
        }

        inline void write_verification_data ( __BIT_LIST_RAW_DATA_TYPE__ * data ) const {
            write_data( data );
        }
        
        inline static bool narrow_feasible ( EdgeCandidatesContainer & container, const __BIT_LIST_RAW_DATA_TYPE__ * const node_data  ) {
    #ifdef __STATS_OUTPUT__
            TreeNodeCondition::NARROWING_OPS++;
    #endif
            const __BIT_LIST_RAW_DATA_TYPE__ * candidate_data = container.stack()->start_ptr();
            const __m128i * __node_frag = reinterpret_cast<const __m128i *>( node_data );
            const uint_fast16_t segments = container.stack()->reg_1();
            const __m128i * __cand_frag = reinterpret_cast<const __m128i *>( candidate_data );
            
            uint_fast16_t i = 0;
            NarrowingDataResult res;
            res.__vec[0] = GLOBAL_EDGE_DIM;
            res.__vec[1] = -container.candidate()->population();

            do {
                res.__sum = _mm_add_epi64( res.__sum, _mm_sad_epu8( __cand_frag[i], _mm_min_epu8( __node_frag[i], __cand_frag[i] ) ));
            } while ( ++i < segments );
            
            return 0 >= res.__vec[0] + res.__vec[1];
        }         
        
        inline static void intermediate_post_action ( EdgeCandidatesContainer & , const __BIT_LIST_RAW_DATA_TYPE__ * const ) {}
        inline static void leaf_post_action ( EdgeCandidatesContainer & , const __BIT_LIST_RAW_DATA_TYPE__ * const ) {}        
        inline static bool intermediate_post_action ( EdgeCandidate &, const __BIT_LIST_RAW_DATA_TYPE__ * const ) {
            return false;
        }
        inline static bool leaf_post_action ( EdgeCandidate &, const __BIT_LIST_RAW_DATA_TYPE__ * const ) {
            return false;
        }

        inline static bool verify_feasible ( EdgeCandidate & candidate, const __BIT_LIST_RAW_DATA_TYPE__ * const node_data ) {
            
    #ifdef __STATS_OUTPUT__
            TreeNodeCondition::VERIFICATION_OPS++;
    #endif 
            
            const __m128i * __node_frag = reinterpret_cast<const __m128i *>( node_data );
            const uint_fast16_t segments = candidate.stack()->reg_1();
            const __m128i * __cand_frag = reinterpret_cast<const __m128i *>( candidate.stack()->start_ptr() );
            
            uint16_t i = 0;
            VerificationDataResult res;
             
            do {            
                res.__vec = _mm_cmpgt_epi8( __cand_frag[i], __node_frag[i] );
            } while ( ++i < segments && res.__cmp_value == 0 );
            
            return res.__cmp_value == 0;
        }
        
        static uint_fast16_t required_alignment () {
            return sizeof( FragmentationSegment );
        }
        
        static uint_fast16_t guaranteed_alignment () {
            return sizeof( FragmentationSegment );
        }

        static uint_fast16_t dynamic_size () {
            return  ( __DD_BLOCK_COUNT__ + 1 ) * ( sizeof( __BIT_LIST_RAW_DATA_TYPE__ ) + sizeof( FragmentationSegment ) );
        }
        
        uint_fast16_t mem_size () const {
            return __frag_data.segment_count() * sizeof( FragmentationSegment );
        }      
        
        const std::string toString () const;
	};
}

#endif // __DD_FRAGMENTATION_CONDITION__
