/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_FREQUENCY_MODIFIER__
#define __DD_FREQUENCY_MODIFIER__

#include <vector>

#include "DDRayRefModifier.h"
#include "DDBitCountVisitor.h"

namespace addibit {

    class FrequencyModifier : public RayRefModifier {

        protected:
        const std::vector<BitPopulationEntry> & mPopulation;

        virtual unsigned long update ( const unsigned long index, const bool value ) const = 0;

        unsigned long getFrequency ( const BitList & map ) const {
            unsigned long frequency = 0;

            for ( unsigned long i = 0; i < mPopulation.size(); i++ ) {
                frequency += update( i,  map.isSet( mPopulation[ i ].position() ) );
            }

            return frequency;
        }

        public:
        FrequencyModifier( const std::vector<BitPopulationEntry> & population ) :
            RayRefModifier(),
            mPopulation( population ) {}

        void modify ( struct RayReference & ref ) const {
            ref.frequency = getFrequency( ref.activeSet );
        } 
    };
}

#endif // __DD_FREQUENCY_MODIFIER__
