
#ifndef __DD_HAMMING_DISTANCE_SPLIT_VISITOR__
#define __DD_HAMMING_DISTANCE_SPLIT_VISITOR__

#include "DDBitSplitVisitor.h"

namespace addibit {

    enum HammingDistanceSplitVisitorSets {
        HAMMING_SPLIT_LOW   = 0,
        HAMMING_SPLIT_HIGH  = 1
    }; 

    class HammingDistanceSplitVisitor : public BitSplitVisitor {

        private:
        BitList mVantagePoint;
        const unsigned int mMinDistance;
        unsigned int mOffset;

        public:
        HammingDistanceSplitVisitor ( const unsigned int depth,
                                      const unsigned int capacity,
                                      const unsigned int minDistance ) :
            BitSplitVisitor( depth, capacity, 2 ),
            mVantagePoint(),
            mMinDistance( minDistance ) {}

        void onStart () {
            BitSplitVisitor::onStart();
            mVantagePoint.clear();
        }

        void onVisit ( const struct RayReference & ref ) {
            BitSplitVisitor::onVisit( ref );             

            if ( unlikely( mVantagePoint.isEmpty() ) ) {
                mVantagePoint = ref.activeSet;
                set( HAMMING_SPLIT_LOW ).insert( ref );      
            } else {
                const BitList hamming_distance_map = ref.activeSet & mVantagePoint;
		        if ( hamming_distance_map.getPopulationCount() > mOffset ) {
			        set( HAMMING_SPLIT_HIGH ).insert( ref );
		        } else {
			        set( HAMMING_SPLIT_LOW ).insert( ref );
		        }
            }
        }

        void onFinish () {
            if ( set( HAMMING_SPLIT_HIGH ).empty() ) {
                mOffset--;
            }
            BitSplitVisitor::onFinish();
        }

        void before ( const RayReferenceContainer & current,
                      RayReferenceContainer * sets ) {

            BitSplitVisitor::before( current, sets );

            mOffset = mMinDistance;    
        }

        bool hasFinished () const {
            return !set( HAMMING_SPLIT_HIGH ).empty() || set( HAMMING_SPLIT_LOW ).size() == 1;
        }  
    };
}

#endif // __DD_HAMMING_DISTANCE_SPLIT_VISITOR__
