/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_INPUT__
#define __DD_INPUT__

#include <fstream>

#include "DDOptions.h"
#include "DDMatrix.h"
#include "DDRankCalculator.h"

//@FIXME Input should be robust!
namespace addibit {
	
	class Input {

		private:
        InputOrdering __ordering;
        
		const Matrix mSystem;
		const Matrix transformator;
		const Matrix transformed; 

        const Matrix getMatrixFromFile ( std::ifstream & input ) const;
		const Matrix getMatrixFromFile ( const char * );
        
		const Matrix applyNemhauserWolsey ( Matrix & matrix ) const;
		const Matrix applyPositiveBaseTransformation ( const Matrix& matrix ) const;

		static Matrix getIdentityMatrix ( const unsigned long dimension ) {
			Matrix identity( dimension, dimension );

			for ( unsigned long i = 0; i < dimension; i++ ) {
				identity.setValue( i, i, 1 );
			}

			return identity;
		}

        /* Find a basis and put it in the first d rows. */
		void reorder_for_basis ( Matrix & system ) const;
		
		/* Reorder the rows according to the ordering in the
		 * configuration file. */
		void reorder_for_computation ( Matrix & ) const;

        void process_options ( Matrix &, std::ifstream & );

		public:
		Input ( const char * );
		Input ( const char *, const InputOrdering );

        InputOrdering ordering () const {
            return __ordering;
        }

		const Matrix & getSystem () const {
			return mSystem;
		}

		const Matrix & getTransformedSystem () const {
			return transformed;
		} 

		const Matrix & getTransformator () const {
			return transformator;
		} 
	};
}

#endif // __DD_INPUT__
