/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_INTERMEDIATE_CACHE_ENTRY_NODE__
#define __DD_INTERMEDIATE_CACHE_ENTRY_NODE__

#include "DDLogging.h"

#include "DDCacheEntryNode.h"
#include "DDCacheEntryContainer.h"

namespace addibit {

    template <class RAY_TYPE, class CACHE_ENTRY_TYPE>
    class IntermediateCacheEntryNode : public CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> {
    
        private:
        CacheEntryContainer<RAY_TYPE, CACHE_ENTRY_TYPE> mSuccessors;
        
        public:
        IntermediateCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                     const uint_fast16_t successors ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data ),
            mSuccessors( successors ) {}
            
        IntermediateCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                     const uint_fast16_t successors,
                                     const uint_fast16_t id ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data, id ),
            mSuccessors( successors ) {}    
            
        IntermediateCacheEntryNode ( CACHE_ENTRY_TYPE * data,
                                     const uint_fast16_t successors,
                                     const uint_fast16_t id,
                                     const uint_fast16_t thread_id ) :
            CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>( data, id, thread_id ),
            mSuccessors( successors ) {}                    
            
        virtual ~IntermediateCacheEntryNode () {}
        
        bool push ( std::shared_ptr<const RAY_TYPE > ray ) {
            bool result = false;
            
            for ( auto & entry : mSuccessors ) {
		        if ( entry->data().proceed( ray ) ) {
		            result |= entry->push( ray );
		        }
		    }
            
            return result;
		}
		
		void insert ( CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * entry, const uint_fast16_t pos ) {
		    mSuccessors[ pos ] = entry;
		}
		
		void enum_successors ( std::deque<CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> * > & buffer ) {
            buffer.insert( buffer.begin(), mSuccessors.begin(), mSuccessors.end() );
		}
        
        virtual const std::string toGraphVizString () const {
            std::stringstream result;

            result << CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE>::toGraphVizString() << "\n";

            for ( auto & entry : mSuccessors ) {
                result << entry->toGraphVizString() << "\n";
		        result << "p" << static_cast<const void*>( this ) << "->" << "p" << static_cast<const void*>( entry ) << ";\n";
		    }      
            
            return result.str();      
        }
    };
}

#endif /* __DD_INTERMEDIATE_CACHE_ENTRY_NODE__ */
