/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_LOGGING__
#define __DD_LOGGING__

#include <iostream>
#include <sstream>
#include <map>
#include <mutex>

#define DD_OUTPUT( message )	\
	std::cout << message << ::std::endl;
		
#define DD_START_OUTPUT( ID ) \
    std::ostringstream ID##_stream; \
    ID##_stream.precision( 2 );
   
#define DD_CONCAT_OUTPUT( ID, message )	\
	ID##_stream << message;   
    
#define DD_FLUSH_OUTPUT( ID )	\
    output_mutex.lock();     \
    std::cout << ID##_stream.str() << std::endl;    \
    output_mutex.unlock();
        
#define DD_CONCAT( message )	\
	std::cout << message;


namespace addibit {
    extern std::mutex output_mutex;
    extern std::mutex plot_mutex;
    extern std::map<std::string, std::stringstream*> loggers;
    extern void DUMP_STRING ( const char * , const std::string & );   
    extern void init_logging ();
}

#define OPEN_LOG( ID ) std::stringstream * _sstream_##ID = new std::stringstream; \
                        loggers[ #ID ] = _sstream_##ID;

#define LOG_DATA( ID, DATA ) *loggers[ #ID ] << DATA;

#define LOG_LINE( ID, LINE )    \
    plot_mutex.lock();  \
    *loggers[ #ID ] << LINE << std::endl;   \
    plot_mutex.unlock();

#define DUMP_LOG( ID, FILE ) std::ofstream ID##_ofstream( FILE ); \
                             ID##_ofstream << loggers[ #ID ]->str(); \
                             ID##_ofstream.close(); \
                             delete loggers[ #ID ]; \
                             loggers.erase( #ID );
                            

#ifndef __DD_DEBUG_LOGGING__
#define DD_LOG_DEBUG( message );
#else
#define DD_LOG_DEBUG( message )							\
	std::cout <<							        \
	  "(" << __FILE__ << "," << __LINE__ << "): "	\
		  << __FUNCTION__ << "(): ";				\
  		std::cout << message << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT(v)	;
#else
#define DD_PRINT(v)							\
	std::cout <<							\
	  "("__FILE__"," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << #v << ": " << (v) << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT_RAY(var)	;
#else
#define DD_PRINT_RAY(var)						\
	std::cout <<							\
	  "("__FILE__"," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << (var->toString()) << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT_VECTOR_ELEMENTARY(var)	;
#else
#define DD_PRINT_VECTOR_ELEMENTARY(var)				\
	std::cout <<							\
	  "(" << __FILE__ << "," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << "[";     \
      for ( unsigned int i = 0; i < var.size(); i++ ) { \
        std::cout << i << ":" << var[i];    \
        if ( i != var.size() - 1 ) {    \
            std::cout << ", ";  \
        }   \
      } \
      std::cout << "]" << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT_VECTOR_OBJECT(var, func)	;
#else
#define DD_PRINT_VECTOR_OBJECT(var, func)				\
	std::cout <<							\
	  "("__FILE__"," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << "[";     \
      for ( unsigned int i = 0; i < var.size(); i++ ) { \
        std::cout << var[i].func();    \
        if ( i != var.size() - 1 ) {    \
            std::cout << ", ";  \
        }   \
      } \
      std::cout << "]" << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT_VECTOR_POINTER(var, func)	;
#else
#define DD_PRINT_VECTOR_POINTER(var, func)				\
	std::cout <<							\
	  "("__FILE__"," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << "[";     \
      for ( unsigned int i = 0; i < var.size(); i++ ) { \
        std::cout << var[i]->func();    \
        if ( i != var.size() - 1 ) {    \
            std::cout << ", ";  \
        }   \
      } \
      std::cout << "]" << ::std::endl << ::std::flush;
#endif

#ifdef __DD_DEBUG__
#define DD_PRINT_MATRIX(var)	;
#else
#define DD_PRINT_MATRIX(var)						\
	std::cout <<							\
	  "("__FILE__"," << __LINE__ << "): "				\
		  << __FUNCTION__ << "(): ";				\
      std::cout << var.getRows() << " x " << var.getColumns() << " matrix:" << ::std::endl << var.toString() << std::endl << ::std::flush;
#endif

#endif // __DD_LOGGING__
