/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDMatrixUtils.h"

using namespace addibit;

MatrixUtils::MatrixUtils ( CalcModule& calcModule ) :
	mCalcModule( calcModule ) {}

const Matrix MatrixUtils::applyPositiveBaseTransformation ( const Matrix& matrix ) {
	Matrix base = matrix.getSubMatrix( 0, matrix.getColumns() );
	
	return applyNemhauserWolsey( base );
}

const Matrix MatrixUtils::applyNemhauserWolsey ( Matrix& matrix ) {

	Matrix C = MatrixUtils::getIdentityMatrix( matrix.getRows() );
	Matrix transformation = MatrixUtils::getIdentityMatrix( matrix.getRows() );
	unsigned long row, column;

	for ( row = 0; row < std::min( matrix.getRows(), matrix.getColumns() ); row++ ) {
	
		for ( column = row + 1; column < matrix.getColumns(); column++ ) {

			if ( matrix.isNonZeroAt( row, column ) ) {
				mCalcModule.calcExendedGCD ( matrix[row][row], matrix[row][column] );

				C.setValue ( row, row, mCalcModule.getRegisterS() );
				C.setValue ( column, row, mCalcModule.getRegisterT() );
				C.setValue ( row, column, -matrix[row][column]/mCalcModule.getResult() );
				C.setValue ( column, column, matrix[row][row]/mCalcModule.getResult() );

				matrix *= C;
				transformation *= C;

				/* Transform C back to identity matrix */
				C.setValue ( row, row, 1 );
				C.setValue ( column, row, 0 );
				C.setValue ( row, column, 0 );
				C.setValue ( column, column, 1 );

			//	std::cout << matrix.toString() << std::endl << std::endl;
			}
		}
		
		if ( matrix.getValue( row, row ) < 0 ) {
			
			C.setValue ( row, row, -1 );
			
			matrix *= C;
			transformation *= C;	

			/* Transform C back to identity matrix */
			C.setValue ( row, row, 1 );
		}

		for ( column = 0; column < row; column++ ) {

			if ( matrix.isNonZeroAt( row, column ) ) {

				C.setValue ( row, column, -( matrix[row][column]/matrix[row][row] + 1 ) );

				matrix *= C;
				transformation *= C;

				/* Transform C back to identity matrix */
				C.setValue ( row, column, 0 );
			}
		} 

		for ( column = 0; column < row; column++ ) {

			if ( matrix.isNonZeroAt( row, column ) ) {

				if ( matrix[row][column] % matrix[row][row] != 0 ) {
					C.setValue ( row, column, ( abs( matrix[row][column]/matrix[row][row]) + 1 ) );
				} else {
					C.setValue ( row, column, abs( matrix[row][column]/matrix[row][row] ) );
				}

				matrix *= C;
				transformation *= C;

				/* Transform C back to identity matrix */
				C.setValue ( row, column, 0 );
			}
		} 
	}

	for ( row = 0; row < std::min( matrix.getRows(), matrix.getColumns() ); row++ ) {

		matrix.setValue( row, row, 1 );

		for ( column = 0; column < row; column++ ) {			

			if ( matrix.isNonZeroAt( row, column ) ) {

				C.setValue( row, column, -matrix[row][column] );

				matrix *= C;
				transformation *= C;

				/* Transform C back to identity matrix */
				C.setValue ( row, column, 0 );
			}
		}
	}	

	return transformation;	
}


