
#ifndef __DD_MOST_AVERAGE_FIRST_DYNAMIC_SAMPLE_CREATOR__
#define __DD_MOST_AVERAGE_FIRST_DYNAMIC_SAMPLE_CREATOR__

#include <vector>
#include <algorithm>

#include "DDDynamicSampleCreator.h"

namespace addibit {

    struct MostAverageFirstComparator {

        private:
        const unsigned long mAverage;

        public:
        MostAverageFirstComparator ( const unsigned long average ) :
            mAverage( average ) {}

        bool operator () ( const struct PopulationEntry & lhs, const struct PopulationEntry & rhs ) const {
            unsigned long lhs_diff = lhs.count > mAverage ? lhs.count - mAverage : mAverage - lhs.count;
            unsigned long rhs_diff = rhs.count > mAverage ? rhs.count - mAverage : mAverage - rhs.count;

            return lhs_diff < rhs_diff;
        }
    };

    class MostAverageFirstDynamicSampleCreator : public DynamicSampleCreator {

        private:
        unsigned long getAverage ( const std::vector<struct PopulationEntry> & stats ) const {
            unsigned long sum = 0;
        
            for ( unsigned long index = 0; index < stats.size(); index++ ) {
                sum += stats[ index ].count;
            }

            return sum / stats.size();
        }

        public:
        MostAverageFirstDynamicSampleCreator( const unsigned int blocks,
                                              const unsigned int minCommonBits,
                                              const float intensityFactor,
                                              const float sampleDifference,
                                              const float stretch ) :
            DynamicSampleCreator( blocks, minCommonBits, intensityFactor, sampleDifference, stretch ) {}   

            void reorder ( std::vector<struct PopulationEntry> & stats ) const {
                struct MostAverageFirstComparator comparator ( getAverage( stats ) );
                sort( stats.begin(), stats.end(), comparator );
            }
    };
}

#endif // __DD_MOST_AVERAGE_FIRST_DYNAMIC_SAMPLE_CREATOR__
