
#ifndef __DD_MULTIPLE_BIT_SPLIT_VISITOR__
#define __DD_MULTIPLE_BIT_SPLIT_VISITOR__

#include "DDBitSplitVisitor.h"

namespace addibit {

    enum MultipleBitSplitVisitorSets {
        MULTI_BIT_SPLIT_SUB_SET     = 0,
        MULTI_BIT_SPLIT_SUPER_SET   = 1
    };

    class MultipleBitSplitVisitor : public BitSplitVisitor {

        private:
        BitList mCurrentMap;
        const unsigned int mMinUnsetBits;
        unsigned int mOffset;

        unsigned int mDeadZeroes;
        unsigned int mOffset_2;

        const double mNarrowingFactor;
        const double mBalanceFactor;

        public:
        MultipleBitSplitVisitor ( const unsigned int depth,
                                  const unsigned int capacity,
                                  const unsigned int minUnsetBits,
                                  const double narrowing,
                                  const double balance ) :
            BitSplitVisitor( depth, capacity, 2 ),
            mCurrentMap(),
            mMinUnsetBits( minUnsetBits ),
            mOffset( 0 ),
            mNarrowingFactor( narrowing ),
            mBalanceFactor( balance ) {}

        void onStart () {
            BitSplitVisitor::onStart();
            mCurrentMap.clear();
        }

        void onVisit ( const struct RayReference & ref ) {
            BitSplitVisitor::onVisit( ref );             

            if ( unlikely( mCurrentMap.isEmpty() ) ) {
                mCurrentMap = ~ref.activeSet;
                if ( mOffset + ref.degeneracy > mCurrentMap.getCapacity() ) {
                    mOffset = mCurrentMap.getCapacity() - ref.degeneracy;
                }
                set( MULTI_BIT_SPLIT_SUB_SET ).insert( ref );      
            } else {
                const unsigned int zero_cut = ref.activeSet.getInvCommonPopCnt( mCurrentMap ) - mDeadZeroes;
		        if ( zero_cut * mNarrowingFactor >= mCurrentMap.getPopulationCount() - mDeadZeroes && zero_cut >= mOffset_2 ) {
			        set( MULTI_BIT_SPLIT_SUB_SET ).insert( ref );
                    mCurrentMap &= ~ref.activeSet;
		        } else {
			        set( MULTI_BIT_SPLIT_SUPER_SET ).insert( ref );
		        }
            }
        }

        void onFinish () {
            if ( !hasFinished() ) {
                mOffset++;
                mOffset_2--;
            } 
            BitSplitVisitor::onFinish();
        }

        void before ( const RayReferenceContainer & current ) {

            BitSplitVisitor::before( current );

            mDeadZeroes = current.union_map().getCapacity() - current.union_map().getPopulationCount();
            mOffset = mMinUnsetBits + current.union_map().getCapacity() - current.union_map().getPopulationCount();
            mOffset_2 = mMinUnsetBits;
        }

        bool hasFinished () const {
            const unsigned long superset_size = set( MULTI_BIT_SPLIT_SUPER_SET ).size();
            const unsigned long subset_size = set( MULTI_BIT_SPLIT_SUB_SET ).size();

            return superset_size > 0 && ( subset_size > superset_size * mBalanceFactor || mOffset_2 < 2 );
        }

        const unsigned int & offset () const {
            return mOffset;
        }

        const BitList & getMap() const {
            return mCurrentMap;
        }    
    };
}

#endif // __DD_MULTIPLE_BIT_SPLIT_VISITOR__
