/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "DDOptions.h"

using namespace addibit;

Options * Options::mCurrent = NULL;
const char Options::COMMENTS_DELIM = '#';
const char Options::ASSIGNMENT_DELIM = '=';

void TreeOptions::set ( const std::string & name, const std::string & value ) {
            
    if ( name == OPT_CONTAINER_TYPE ) {
        Options::write_value( value, mTreeType, TreeTypeNames, TreeTypeNamesSize );
    } else if ( name == OPT_BIT_SELECT_STRATEGY ) {
        Options::write_value( value, mBPT_Strategy, BP_Tree_BitSelectionStrategyNames, BP_Tree_BitSelectionStrategyNamesSize );
    } else if ( name == OPT_FRAGMENTATION ) {
        Options::write_value( value, mFBPT_Fragmentation );
    } else if ( name == OPT_MAX_TREE_DEPTH ) {
        Options::write_value( value, mMaxTreeDepth );
    }  else if ( name == OPT_LEAF_CAPACITY ) {
        Options::write_value( value, mLeafCapacity );
    } else if ( name == OPT_DATA_SORT ) {
        Options::write_value( value, mDataSort, SortTypeNames, SortTypeNamesSize );
    } else if ( name == OPT_BITS_PER_SPLIT ) {
        Options::write_value( value, mBitsPerSplit );
    } else if ( name == OPT_VANTAGE_POINT_DISTANCE_COEFF ) {
        Options::write_value( value, mVantagePointDistanceCoefficient );
    } else if ( name == OPT_VANTAGE_POINT_POPULATION ) {
        Options::write_value( value, mVantagePointPopulation );
    } else if ( name == OPT_DYNAMIC_NARROWING_FACTOR ) {
        Options::write_value( value, mDynamicNarrowingFactor );
    } else if ( name == OPT_DYNAMIC_BALANCE_FACTOR ) {
        Options::write_value( value, mDynamicBalanceFactor );
    } else if ( name == OPT_UNIQUE_VANTAGE_POINT ) {
        Options::write_value( value, mUniqueVantagePoint );
    }
}

const std::string Options::getCaptionBlock ( const std::string text ) const {
    std::stringstream result;
    const std::string line( 83, '-' );   

    result << line << std::endl
           << "|" << std::string( 40 - text.length()/2, ' ' ) << text << std::string( 41 - ( text.length()/2 + text.length() % 2 ), ' ' ) << "|" << std::endl
           << line;

    return result.str();
}

const std::string TreeOptions::toString () const {
    std::stringstream result;

    result << " -> " << OPT_CONTAINER_TYPE << " = " << TreeTypeNames[ mTreeType ] << std::endl;
    
    if ( mTreeType == BIT_PATTERN_TREE || mTreeType == EXTENDED_BIT_PATTERN_TREE || mTreeType == FRAGMENTATION_TREE ) {
        result << " -> " << OPT_BIT_SELECT_STRATEGY << " = " << BP_Tree_BitSelectionStrategyNames[ mBPT_Strategy ] << std::endl;
    }
    
    if ( mTreeType == FRAGMENTATION_TREE ) {
    
         result << " -> " << OPT_FRAGMENTATION << " = ";
        if ( mFBPT_Fragmentation > 1 ) {
            result << mFBPT_Fragmentation << " bits";
        } else {
            result << "none";
        }
        result << std::endl;
    }

    if ( mTreeType == BINARY_DYNAMIC_VANTAGE_POINT ) {
        result << " -> " OPT_UNIQUE_VANTAGE_POINT << " = " << mUniqueVantagePoint << std::endl;
    }    
    
    if ( mTreeType != ARRAY ) {
    
        result << " -> " << OPT_MAX_TREE_DEPTH << " = " << mMaxTreeDepth << std::endl
               << " -> " << OPT_LEAF_CAPACITY << " = " << mLeafCapacity << std::endl
               << " -> " << OPT_DATA_SORT << " = " << SortTypeNames[ mDataSort ];
    }
    
    return result.str();
}

const std::string Options::toString () const {
    std::stringstream result;

    result << getCaptionBlock( "GENERAL SETTINGS" ) << std::endl
           << " -> " << OPT_ADJACENCY_TEST << " = " << AdjacencyTestNames[ mAdjacencyTest ] << std::endl
           << " -> " << OPT_OUTPUT_FORMAT << " = " << OutputFormatNames[ mOutputFormat ] << std::endl
           << " -> " << OPT_PLOT_FILE << " = " << plot_file() << std::endl
           << " -> " << OPT_THREAD_EXPONENT << " = " << thread_exponent() << std::endl
           << " -> " << OPT_INPUT_ORDERING << " = " << InputOrderingNames[ mInputOrdering ] << std::endl
           << " -> " << OPT_NARROWING_DEG_THRESHOLD << " = " << narrowing_degeneracy_threshold() << std::endl
           << " -> " << OPT_VERIFICATION_DEG_THRESHOLD << " = " << verification_degeneracy_threshold() << std::endl
           << getCaptionBlock( SEC_NARROWING_LD_TREE_LD_INPUT ) << std::endl
           << mNarrowingLDInputLDTree.toString() << std::endl
           << getCaptionBlock( SEC_NARROWING_HD_TREE_LD_INPUT ) << std::endl
           << mNarrowingLDInputHDTree.toString() << std::endl
           << getCaptionBlock( SEC_NARROWING_HD_TREE_HD_INPUT ) << std::endl
           << mNarrowingHDInputHDTree.toString() << std::endl;

    if ( mAdjacencyTest == COMBINATORIAL_TEST ) {
        result << getCaptionBlock( SEC_VERIFICATION_LD_TREE ) << std::endl
               << mVerificationLDTree.toString() << std::endl
               << getCaptionBlock( SEC_VERIFICATION_HD_TREE ) << std::endl
               << mVerificationHDTree.toString() << std::endl;
    }

    return result.str();
}

void Options::set ( const std::string & name, const std::string & value ) {
    
    if ( name == OPT_PLOT_FILE ) {
        mPlotFile = value;
    } else if ( name == OPT_OUTPUT_FORMAT ) {
        write_value( value, mOutputFormat, OutputFormatNames, OutputFormatNamesSize );
    } else if ( name == OPT_ADJACENCY_TEST ) {
        write_value( value, mAdjacencyTest, AdjacencyTestNames, AdjacencyTestNamesSize );
    } else if ( name == OPT_THREAD_EXPONENT ) {
        write_value( value, mThreadExponent );
    } else if ( name == OPT_INPUT_ORDERING ) {
        write_value( value, mInputOrdering, InputOrderingNames, InputOrderingNamesSize );
    } else if ( name == OPT_NARROWING_DEG_THRESHOLD ) {
        write_value( value, mNarrowingDegeneracyThreshold );
    } else if ( name == OPT_VERIFICATION_DEG_THRESHOLD ) {
        write_value( value, mVerificationDegeneracyThreshold );
    }
}

bool Options::readSettingsFile ( const char * file ) {
    std::ifstream settings( file );
    const bool good = settings.good();

	if ( good ) {
        std::string line("");

        TreeOptions * current_tree = NULL;

        while( std::getline( settings, line )) {
            remove_comments( line, COMMENTS_DELIM );
            trim( line );
            
             if ( line == SEC_GENERAL_SETTINGS ) {
                current_tree = NULL;
             } else if ( line == SEC_NARROWING_LD_TREE_LD_INPUT ) {
                current_tree = &mNarrowingLDInputLDTree;
             } else if ( line == SEC_NARROWING_HD_TREE_LD_INPUT ) {
                current_tree = &mNarrowingLDInputHDTree;
             } else if ( line == SEC_NARROWING_HD_TREE_HD_INPUT ) {
                current_tree = &mNarrowingHDInputHDTree;
             } else if ( line == SEC_VERIFICATION_LD_TREE ) {
                current_tree = &mVerificationLDTree;
             } else if ( line == SEC_VERIFICATION_HD_TREE ) {
                current_tree = &mVerificationHDTree;
             } else {
                std::istringstream is_line( line );
                std::string key;

                if ( std::getline( is_line, key, ASSIGNMENT_DELIM )) {
                    std::string value;
                    if ( std::getline( is_line, value ) ) {
                        trim( key );
                        trim( value );

                        if ( current_tree != NULL  ) {
                            current_tree->set( key, value );
                        } else {
                            set( key, value );
                        }
                     }
                 }
             }
         }
    }

    settings.close();
    return good;
}
