/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_PART_TEST_TRAVERSE_VISITOR__
#define __DD_PART_TEST_TRAVERSE_VISITOR__

#include "DDPreliminaryTraverseVisitor.h"
#include "DDDualSetRay.h"
#include "DDDualSetRayCacheEntry.h"
#include "DDLinearDependenceCache.h"
#include "DDFaceProver.h"
#include "DDDualSetFace.h"

namespace addibit {

    class PartTestTraverseVisitor : public PreliminaryTraverseVisitor<DualSetRay, DualSetRayCacheEntry>, public EdgeCandidatesContainerVisitor<DualSetRay> {

        private:
        LinearDependenceCache & mDependenceCache;
        FaceProver & mProver;
        
        protected:
        inline void examine ( const DualSetRay * const positive,
                              const DualSetRay * const negative ) {
        
		    if ( mProver.isFace ( positive, negative ) ) {
			    DualSetFace face( positive, negative );
			    face.setPrimaryActiveSet( mProver.getRetainedPrimaryBits(), mPlane );
			    face.setSecondaryActiveSet( mProver.getProvenSecondaryBits() );
        
			    if ( mCacheNode.push( std::shared_ptr<const DualSetRay>( face.getIntersection( mPlane, GLOBAL_MATRIX_ROWS, mDependenceCache ) ) ) ) {
                    inc_extremal();
                } else {
                    inc_verified();
                }
		    }                                        
        }
    
        virtual void check_candidates ( const DualSetRay * ) = 0;
    
        public:
        PartTestTraverseVisitor( const TreeNode * const tree,
                                 const RayWrapperContainer<DualSetRay> & container,
                                 const uint_fast16_t stack_size,
                                 const uint_fast16_t plane,
                                 CacheEntryNode<DualSetRay, DualSetRayCacheEntry> & node,
                                 LinearDependenceCache & dependencies,
                                 FaceProver & prover ) :
            PreliminaryTraverseVisitor<DualSetRay, DualSetRayCacheEntry>( tree, container, stack_size, plane, node ),
            mDependenceCache( dependencies ),
            mProver( prover ) {}

        using PreliminaryTraverseVisitor<DualSetRay, DualSetRayCacheEntry>::narrow;

        inline void process_ray ( const DualSetRay & ray ) {
            narrow( ray.getActiveSet() );
            check_candidates( &ray );
        }

        ~PartTestTraverseVisitor () {}
    };
    
    class PositivePartTestTraverseVisitor : public PartTestTraverseVisitor {
    
        protected:
        inline void check_candidates ( const DualSetRay * ray ) {
            mCandidatesContainer.verify<DualSetRay, PositivePartTestTraverseVisitor>( ray, *this );
        }
    
        public:
        PositivePartTestTraverseVisitor ( const TreeNode * const tree,
                                          const RayWrapperContainer<DualSetRay> & container,
                                          const uint_fast16_t stack_size,
                                          const uint_fast16_t plane,
                                          CacheEntryNode<DualSetRay, DualSetRayCacheEntry> & node,
                                          LinearDependenceCache & dependencies,
                                          FaceProver & prover ) :
            PartTestTraverseVisitor( tree, container, stack_size, plane, node, dependencies, prover ) {}
        
        void verify_edge ( const DualSetRay * const negative, const AdjacencyCandidate * const candidate ) {            
            examine( reinterpret_cast<const DualSetRay *>( candidate->id() ), negative );
        }
    };
    
    class NegativePartTestTraverseVisitor : public PartTestTraverseVisitor {
    
        protected:
        inline void check_candidates ( const DualSetRay * ray ) {
            mCandidatesContainer.verify<DualSetRay, NegativePartTestTraverseVisitor>( ray, *this );
        }    
    
        public:
        NegativePartTestTraverseVisitor ( const TreeNode * const tree,
                                          const RayWrapperContainer<DualSetRay> & container,
                                          const uint_fast16_t stack_size,
                                          const uint_fast16_t plane,
                                          CacheEntryNode<DualSetRay, DualSetRayCacheEntry> & node,
                                          LinearDependenceCache & dependencies,
                                          FaceProver & prover ) :
            PartTestTraverseVisitor( tree, container, stack_size, plane, node, dependencies, prover ) {}
        
        void verify_edge ( const DualSetRay * const positive, const AdjacencyCandidate * const candidate ) {
            examine( positive, reinterpret_cast<const DualSetRay *>( candidate->id() ) );
        }
    };    
}

#endif // __DD_PART_TEST_TRAVERSE_VISITOR__
