/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_POPULATION_MODIFIER__
#define __DD_POPULATION_MODIFIER__

#include <vector>

#include "DDRayRefModifier.h"
#include "DDBitSelectStrategy.h"
#include "DDBitSelector.h"

namespace addibit {

    class PopulationModifier : public RayRefModifier {

        private:
        BitSelector mSelector;

        const unsigned int mConditionS1;
        const unsigned int mConditionS2;

        const unsigned int mFragments;
        const BitList * mRange;

        const BitList * getSamplesRange ( const unsigned int n,
                                          const unsigned int range ) {

            BitList * result = new BitList[ range ];
            
            result[ 0 ] = mSelector.getNextNBits( n );

            for ( unsigned int i = 1; i < range; i++ ) {
                result[ i ] = result[ i - 1 ];
                result[ i ].setBit( mSelector.getNextBit() );
            }

            return result;
        }   

        public:
        PopulationModifier( const BitSelectStrategy & strategy,
                            const unsigned int bits ) :
            RayRefModifier(),
            mSelector( strategy ),
            mConditionS1( bits / 2 ),
            mConditionS2( bits - mConditionS1 - 1 ),
            mFragments( mSelector.getTotalBits() - bits ),
            mRange( getSamplesRange( mConditionS1 + 1, mFragments ) ) {}

        ~PopulationModifier () {
            delete[] mRange;
        }

        void modify ( struct RayReference & ref ) const {

            unsigned int step = 0;

            while ( mRange[ step ].getCommonPopCnt( ref.activeSet ) <= mConditionS1 ) {
                step++;
            }
            ref.s_1 = step++;

            while ( step < mFragments && mRange[ step ].getInvCommonPopCnt( ref.activeSet ) > mConditionS2 ) {
                step++;
            }
            ref.s_2 = step;
        } 
    };
}

#endif // __DD_POPULATION_MODIFIER__
