/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_PRELIMINARY_TRAVERSE_VISITOR__
#define __DD_PRELIMINARY_TRAVERSE_VISITOR__

#include <deque>
#include <vector>

#include "DDTreeNode.h"
#include "DDRayWrapperVisitor.h"
#include "DDCacheEntryNode.h"
#include "DDCacheEntry.h"

namespace addibit {

    template <class RAY_TYPE, class CACHE_ENTRY_TYPE>
    class PreliminaryTraverseVisitor : public RayWrapperVisitor<RAY_TYPE> {

        private:        
        uint_fast64_t mNarrowed;
        uint_fast64_t mVerified;
        uint_fast64_t mExtremal;

        protected:
        const uint_fast16_t mPlane;
        const uint_fast16_t mPlaneCoeffOffset;
        CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> & mCacheNode;

        const TreeNode * const mTree;
        const RayWrapperContainer<RAY_TYPE> & mContainer;
        
        EdgeCandidatesContainer mCandidatesContainer;
        
        inline bool are_trivially_adjacent ( const RAY_TYPE * const r_1, const RAY_TYPE * const r_2, const AdjacencyCandidate * const face ) const {
            return std::min( r_1->getActiveSet().getPopulationCount(), r_2->getActiveSet().getPopulationCount() ) == face->population() + 1;
        }

        inline void inc_narrowed ( const uint_fast64_t number ) {
            mNarrowed += number;
        }
        
        inline void inc_verified () {
            ++mVerified;
        }
        
        inline void inc_extremal () {
            inc_verified();
            ++mExtremal;
        }
        
        inline void narrow ( const BitList & candidate_map ) {  
            mCandidatesContainer.load( candidate_map, mCacheNode.data().pattern() );
            mTree->narrow( mCandidatesContainer );
            inc_narrowed( mCandidatesContainer.size() );             
        }

        public:
        PreliminaryTraverseVisitor( const TreeNode * const tree,
                                    const RayWrapperContainer<RAY_TYPE> & container,
                                    const uint_fast16_t stack_size,
                                    const uint_fast16_t plane,
                                    CacheEntryNode<RAY_TYPE, CACHE_ENTRY_TYPE> & node ) :
            mNarrowed( 0 ),
            mVerified( 0 ),
            mExtremal( 0 ),
            mPlane( plane ),
            mPlaneCoeffOffset( node.data().plane_coeff_offset() ),
            mCacheNode( node ),
            mTree( tree ),
            mContainer( container ),
            mCandidatesContainer( container.size(), stack_size ) {}          

        inline uint_fast64_t verified () const  {
            return mVerified;
        }
        
        inline uint_fast64_t narrowed () const {
            return mNarrowed;
        }
        
        inline uint_fast64_t extremal () const {
            return mExtremal;
        }      
        
        uint_fast16_t plane () const {
            return mPlane;
        }  

        virtual ~PreliminaryTraverseVisitor () {}
    };
}

#endif // __DD_PRELIMINARY_TRAVERSE_VISITOR__
