/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RANK_CALCULATOR__
#define __DD_RANK_CALCULATOR__

#include "DDBitList.h"
#include "DDCalculationMatrix.h"

namespace addibit {

	class RankCalculator : public CalculationMatrix {

        private:
        void copy_row ( const uint_fast16_t src_index,
                        const uint_fast16_t dst_index,
                        const Matrix & matrix ) {
            
            const CalculationVector & matrix_vec = matrix.getRow( src_index );
            CalculationVector & calculator_vec = getRow( dst_index );

            calculator_vec = matrix_vec;            
        }

		public:
		RankCalculator ( const uint_fast16_t rows, const uint_fast16_t columns );
		virtual ~RankCalculator() {}

        inline void load ( const Matrix & matrix, const BitList & rowMap ) {
            load( matrix, rowMap.data() );
        }
        
        inline void load_first_row ( const Matrix & matrix ) {
            copy_row( 0, 0, matrix );
            setRows( 1 );
            setColumns( matrix.getColumns() );
        }

		void load ( const Matrix& matrix, const __BIT_LIST_RAW_DATA_TYPE__ * const rowMap ) {
            uint_fast16_t last_row = 0;

            for ( uint_fast16_t block = 0, prefix = 0; block < __DD_BLOCK_COUNT__; block++, prefix += __BLOCK_SIZE__ ) {
                __BIT_LIST_RAW_DATA_TYPE__ block_data = rowMap[ block ];

			    while ( block_data ) {
                    copy_row( prefix + __builtin_ctzl( block_data ), last_row++, matrix );
				    __REMOVE_LS_BIT( block_data );
			    }
            }


/*
			for ( unsigned long row = 0; row < matrix.getRows(); row++ ) {
				if ( rowMap.isSet( row ) ) {

					for ( unsigned long column = 0; column < matrix.getColumns(); column++ ) {
						setValue( last_row, column, matrix[row][column] );
					}
					last_row++;
				}
			}
*/
			setRows( last_row );
			setColumns( matrix.getColumns() );
		}

        void addRow ( const CalculationVector & vec ) {

            unsigned long last_row = getRows();
            setRows( last_row + 1 );

		    for ( unsigned long column = 0; column < vec.getDimension(); column++ ) {
			    setValue( last_row, column, vec[column] );
			}
        }

        void removeRow () {
            setRows( getRows() - 1 );
        }

        void overwriteLastRow ( const CalculationVector & vec ) {

		    for ( uint_fast16_t column = 0; column < vec.getDimension(); column++ ) {
			    setValue( getRows() - 1, column, vec[ column ] );
			}
        }

		bool hasMinimumRank ( const uint_fast16_t rank ) {
			uint_fast16_t row, column;
			int_fast16_t clearance = getColumns() - rank;

			for ( row = 0, column = 0; row < getRows() && column < getColumns(); column++ ) {
				if ( findPivot( row, column ) ) {
                    eliminateColumnDown( row, getRows(), column );
					row++;
				} else if ( unlikely( --clearance < 0 ) ) {
					return false;
				}
			}

			return true;
		}

		bool hasMinimumRank ( const Matrix& matrix, const BitList& rowMap, const unsigned long rank ) {
			load( matrix, rowMap );
			return hasMinimumRank( rank );
		}
	};
}

#endif // __DD_RANK_CALCULATOR__
