/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY__
#define __DD_RAY__

#include "DDBitList.h"
#include "DDVector.h"
#include "DDRepresentationVector.h"
#include "DDCalculationVector.h"
#include "DDUtilsFactory.h"

#include <string.h>

#include <gmp.h>
#include <gmpxx.h>

#include "DDLogging.h"

namespace addibit {

	class Ray {

		private:
		const uint_fast16_t __plane_coeff_offset;
		mpz_class * const __plane_coeff_mem;
        
        inline mpz_class * alloc_coeff_mem ( const uint_fast16_t size ) const {
            return ( new mpz_class[ size ] ) - __plane_coeff_offset;
        }
        
        inline void free_coeff_mem () {
            delete[] ( __plane_coeff_mem + __plane_coeff_offset );
        }

		public:
		
		Ray ( mpz_class * const coeffs ) :
			__plane_coeff_offset( GLOBAL_RAY_DIM ),
			__plane_coeff_mem( coeffs - GLOBAL_RAY_DIM ) {}

		Ray ( const uint_fast16_t offset,
              const uint_fast16_t coeff_size ) :
			__plane_coeff_offset( offset ),
			__plane_coeff_mem( alloc_coeff_mem( coeff_size ) ) {}

		virtual ~Ray () {
			free_coeff_mem();
		}
        
        inline uint_fast16_t plane_coeff_offset () const {
            return __plane_coeff_offset;
        }

		inline const mpz_class & getCachedPlaneCoeff ( const uint_fast16_t plane ) const {
			return __plane_coeff_mem[ plane ];
		}

		inline mpz_class & getCachedPlaneCoeff ( const uint_fast16_t plane ) {
			return __plane_coeff_mem[ plane ];
		}

		inline void setCachedPlaneCoeff ( const uint_fast16_t plane, const mpz_class & value ) {
			__plane_coeff_mem[ plane ] = value;
		}
        
		inline void setCachedPlaneCoeff ( const uint_fast16_t plane, const int_fast64_t value ) {
			__plane_coeff_mem[ plane ] = value;
		}        

		inline bool isInside ( const uint_fast16_t plane ) const {
			return getCachedPlaneCoeff ( plane ) > 0;
		}
		
		inline bool isOutside ( const uint_fast16_t plane ) const {
			return getCachedPlaneCoeff ( plane ) < 0;
		}
		
		inline bool isOnThePlane ( const uint_fast16_t plane ) const {
			return getCachedPlaneCoeff ( plane ) == 0;
		}

		const std::string toString ( const uint_fast16_t ) const;

	};
}

#endif // __DD_RAY__
