/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_REF_POPULATION_MAPPING__
#define __DD_RAY_REF_POPULATION_MAPPING__

#include "DDRayRefVisitor.h"
#include "DDBitSelectStrategy.h"

namespace addibit {

    class RayRefPopulationMapping : public RayRefVisitor {
    
        private:
        unsigned int * mMapping;
        
        const BitList * mRange;    
        const unsigned int mLimitBits;    
        const unsigned int mMinCommonBits;
        
        const BitList * getSamplesRange ( const BitSelectStrategy &, const unsigned int, const unsigned int ) const;
    
        public:
        RayRefPopulationMapping ( const BitSelectStrategy & strategy,
                                  const unsigned int lbits,
                                  const unsigned int mbits ) :
            mMapping( NULL ),
            mRange( getSamplesRange( strategy, lbits, mbits ) ),
            mLimitBits( lbits ),
            mMinCommonBits( mbits ) {}
        
        ~RayRefPopulationMapping () {
            delete[] mRange;
            if ( mMapping ) {
                delete[] mMapping;
            }
        }
        
        void init ( const unsigned int rcount ) {
            mMapping = new unsigned int[ rcount ];
        }        
        
        void onVisit ( const RayReference & );
        
        unsigned int operator[] ( const unsigned int id ) const {
            return mMapping[ id ];
        }
        
        const BitList & getFragment ( const unsigned int pos ) const {
            return mRange[ pos ];
        }
        
        unsigned int getConditionalLimit () const {
            return mMinCommonBits - mLimitBits;
        }
    };
}

#endif /* __DD_RAY_REF_POPULATION_MAPPING__ */
