/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_RAY_WRAPPER_CONTAINER__
#define __DD_RAY_WRAPPER_CONTAINER__

#include <deque>
#include <memory>

#include "DDRayReferenceContainer.h"

namespace addibit {

    template<class RAY_TYPE>
    class RayWrapperContainer {
    
        private:
        std::deque<std::shared_ptr<const RAY_TYPE > > mRays;
        
        public:       
        
        inline void insert ( std::shared_ptr<const RAY_TYPE> ray ) {
            mRays.push_back( ray );
        }    
        
        inline const RAY_TYPE * ray ( const uint_fast32_t id ) const {
            return mRays.at( id ).get();
        }    
        
        inline uint_fast32_t size () const {
            return mRays.size();
        }
        
        inline bool empty () const {
            return mRays.empty();
        }
        
        template<class VISITOR> 
        void iterate ( VISITOR & visitor ) const {
            
            visitor.onStart( size() );
            for ( auto & ray : mRays ) {                 
                visitor.process_ray( *ray );
            }
            visitor.onFinish();
        }           
        
        RayReferenceContainer export_refs ( const BitList & pattern) const {
            RayReferenceContainer container( new RayReference[ mRays.size() ], mRays.size() );
            uint_fast32_t id = 0;
            
            for ( auto & ray : mRays ) {                
                container.insert( ray->getActiveSet() & pattern, id++, ( uintptr_t ) ray.get() );
            }
            
            return container;
        }  
        
        template<class Comparator>
        void sort ( Comparator comp ) { /* @FIXME */}
    };
}

#endif /* __DD_RAY_WRAPPER_CONTAINER__ */
