/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_BIT_SPLIT_VISITOR__
#define __DD_SINGLE_BIT_SPLIT_VISITOR__

#include "DDBitSplitVisitor.h"
#include "DDBitSelectStrategy.h"
#include "DDBitSelector.h"

#include "DDContainerBitPatternMapping.h"
#include "DDContainerPopulationMapping.h"

#define BP_TREE_SUB_SET     0
#define BP_TREE_SUPER_SET   1

namespace addibit {

    class SingleBitSplitVisitor : virtual public BitSplitVisitor {

        private:
        BitSelector mSelector;
        uint_fast16_t mSplitBit;

        protected:        
        ContainerBitPatternMapping mContainerMapping;
        ContainerPopulationMapping mPopulationMapping;

        inline void setSplitBit ( const uint_fast16_t bit ) {
            mSplitBit = bit;
        }

        inline BitSelector & selector () {
            return mSelector;
        }

        inline const BitSelector & selector () const {
            return mSelector;
        }        

        void registerContainer ( const unsigned int ) {
            mContainerMapping.insert_empty();
            mPopulationMapping.insert_empty();
        }
        
        unsigned int getTargetContainer ( const RayReference & ref ) {
            const unsigned int cindex = ref.activeSet.isSet( mSplitBit ) ? BP_TREE_SUPER_SET : BP_TREE_SUB_SET;
            mContainerMapping.update( set( cindex ).id(), ref.activeSet );
            mPopulationMapping.update( set( cindex ).id(), ref.degeneracy );
            return cindex;
        }

        public:
        SingleBitSplitVisitor( const unsigned int depth,
                               const unsigned int capacity,
                               const BitSelectStrategy & strategy ) :
            BitSplitVisitor( depth, capacity, 2 ),
            mSelector( strategy ),
            mSplitBit( 0 ) {}
 
        void init ( const RayReferenceContainer & container ) {
            mContainerMapping.reset();
            mPopulationMapping.reset();
            
            BitSplitVisitor::init( container );
                        
            container.iterate( mContainerMapping );
        } 
 
        void before ( const RayReferenceContainer & current ) {
            BitSplitVisitor::before( current );
            const unsigned int cid = current.id();
            mSplitBit = mSelector.getNextBit( mContainerMapping.union_map( cid ) - mContainerMapping.cut_map( cid ) );
        }

        void after () {
            BitSplitVisitor::after();
            mSelector.revert();
        }

        bool can_proceed ( const unsigned int index ) const {
            const unsigned int cid = set( index ).id();
            return BitSplitVisitor::can_proceed( index ) && mSelector.checkNextBit( mContainerMapping.union_map( cid ) - mContainerMapping.cut_map( cid ) );
        }
        
        void refineCondition ( const uint_fast32_t index,
                               TreeNodeCondition & cond ) const {
            
            const uint_fast32_t container_id = set( index ).id();
            cond = TreeNodeCondition( BitList( mContainerMapping.union_map( container_id ) ) );
        }        

        void refineCondition ( const uint_fast32_t index,
                               ExtendedBitPatternCondition & cond ) const {

            const uint_fast32_t container_id = set( index ).id();
            cond = ExtendedBitPatternCondition( BitList( mContainerMapping.union_map( container_id ) ),
                                                mPopulationMapping[ container_id ].ub(),
                                                BitList( mContainerMapping.cut_map( container_id ) ) );
        }        
    };
}

#endif // __DD_SINGLE_BIT_SPLIT_VISITOR__
