/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_SET_RAY__
#define __DD_SINGLE_SET_RAY__

#include "DDBitList.h"
#include "DDRay.h"

#include <string.h>

#include <gmp.h>
#include <gmpxx.h>

namespace addibit {

	class SingleSetRay : public Ray {

		private:
		BitList mActiveSet;

		public:
		SingleSetRay ( mpz_class* const coeffs, BitList && active_set ) :
			Ray( coeffs ),
			mActiveSet( std::move( active_set ) ) {}
		
		SingleSetRay ( const SingleSetRay * const positive, const SingleSetRay * const negative, const uint_fast16_t plane, const uint_fast16_t total_planes, const uint_fast16_t offset ) :
			Ray( offset, total_planes - offset ),
			mActiveSet( positive->getActiveSet() & negative->getActiveSet(), plane ) {}		

		~SingleSetRay () {}

		const BitList & getActiveSet () const {
			return mActiveSet;
		}

		const BitList & getMinimalActiveSet () const {
			return mActiveSet;
		}

        bool degenerate () const {
            return mActiveSet.getPopulationCount() > GLOBAL_RAY_MIN_FACETS;
        }
        
        uint_fast16_t deg_degree () const {
            return mActiveSet.getPopulationCount() - GLOBAL_RAY_MIN_FACETS;
        }

		inline void setCachedPlaneCoeff ( const uint_fast16_t plane, const mpz_class & value ) {
			Ray::setCachedPlaneCoeff( plane, value );
            if ( mpz_cmp_ui ( value.get_mpz_t(), 0 ) == 0 ) {
                mActiveSet.setBit( plane );
            }
		}

		void setActiveBit ( const uint_fast16_t position ) {
			mActiveSet.setBit( position );
		}

		const std::string toString () const;
	};
}

#endif // __DD_SINGLE_SET_RAY__
