/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_SINGLE_SET_RAY_GENERATOR__
#define __DD_SINGLE_SET_RAY_GENERATOR__

#include "DDExtremeRayGenerator.h"

#include "DDMatrix.h"
#include "DDSingleSetRay.h"

#include "DDExtremeRayUncompressor.h"
#include "DDCacheEntry.h"

namespace addibit {

    template<class CACHE_ENTRY_TYPE>
	class SingleSetRayGenerator : public ExtremeRayGenerator<SingleSetRay, CACHE_ENTRY_TYPE> {

        protected:
        void calculateBasis ( CacheEntryNode<SingleSetRay, CACHE_ENTRY_TYPE> * const );

		public:
		SingleSetRayGenerator ( const Matrix & system ) :
            ExtremeRayGenerator<SingleSetRay, CACHE_ENTRY_TYPE>( system ) {}
            
		SingleSetRayGenerator ( const Matrix & system,
		                        CacheEntryNode<SingleSetRay, CACHE_ENTRY_TYPE> * const node ) :
            ExtremeRayGenerator<SingleSetRay, CACHE_ENTRY_TYPE>( system, node ) {}   
            
        virtual ~SingleSetRayGenerator () {}
                         
	};
	
	template<class CACHE_ENTRY_TYPE>
	void SingleSetRayGenerator<CACHE_ENTRY_TYPE>::calculateBasis ( CacheEntryNode<SingleSetRay, CACHE_ENTRY_TYPE> * const node ) {
	
	    const Matrix & system = ExtremeRayGenerator<SingleSetRay, CACHE_ENTRY_TYPE>::mSystem;
	
        ExtremeRayUncompressor decompressor( system.getRows(), system.getColumns() );

	    for ( uint_fast16_t row = system.equalities(); row < system.getColumns(); ++row ) {

		    BitList surface_map( __BIT_LIST_ALLOC_SPACE__, 0 );
		    RepresentationVector vector( system.getColumns() );
		    mpz_class* coeffs = new mpz_class[ system.getRows() - system.getColumns() ];

            surface_map.setBits( 0, system.getColumns() );
		    surface_map.unsetBit( row );

            decompressor.calcExtremeRay( system, surface_map, vector );

		    if ( vector * system[row] < 0 ) {
			    vector.negate();
		    }

		    for ( uint_fast16_t index = system.getColumns(); index < system.getRows(); ++index ) {
		        mpz_class & value = coeffs[ index - system.getColumns() ];
		    
			    vector.calcScalarProduct( system.getRow( index ), value );
			    if ( unlikely( value == 0 ) ) {
				    surface_map.setBit( index );
			    }
		    }

		   node->push( std::shared_ptr<const SingleSetRay>( new SingleSetRay( coeffs, std::move( surface_map ) ) ) );
	    }	
	}
}

#endif // __DD_SINGLE_SET_RAY_GENERATOR_
