/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_SINGLE_SET_TRAVERSE_VISITOR__
#define __DD_SINGLE_SET_TRAVERSE_VISITOR__

#include "DDPreliminaryTraverseVisitor.h"

#include "DDSingleSetRay.h"
#include "DDCalcModule.h"

namespace addibit {

    template<class CACHE_ENTRY_TYPE>
    class SingleSetTraverseVisitor : public PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE> {

        private:
        CalcModule mCalcModule;
        
        /* BEGIN ==== protected members from superclass ==== BEGIN */
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::mPlane;
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::mPlaneCoeffOffset;
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::mTree;
        
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::inc_narrowed;
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::inc_verified;
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::inc_extremal;
        /*  END  ==== protected members from superclass ====  END */
        
        
		inline void init_single_coeff ( const Ray * const positive,
		                                const Ray * const negative,
		                                const uint_fast16_t position,
                                        SingleSetRay * const new_ray ) {
		
            if ( unlikely( negative->isOnThePlane( position ) && positive->isOnThePlane( position ) ) ) {
                new_ray->setCachedPlaneCoeff( position, 0 );
            } else {
                mCalcModule.loadRegisterS( mCalcModule.getRegisterP() );
                mCalcModule.multiplyRegisterS( negative->getCachedPlaneCoeff( position ) ); 
            
                mCalcModule.loadRegisterT( mCalcModule.getRegisterQ() );
                mCalcModule.multiplyRegisterT( positive->getCachedPlaneCoeff( position ) );	
                
                mCalcModule.subtractRegisterS( mCalcModule.getRegisterT() );	

                new_ray->setCachedPlaneCoeff( position, mCalcModule.getRegisterS() ); 
            } 		                                    
		}
		
		inline void prepare_utils ( const Ray * const positive,
		                            const Ray * const negative,
		                            const uint_fast16_t plane ) {
		                            
			mCalcModule.calcGCD( positive->getCachedPlaneCoeff( plane ), negative->getCachedPlaneCoeff( plane ) );

			mCalcModule.loadRegisterP( positive->getCachedPlaneCoeff( plane ) );
			mCalcModule.divideRegisterP( mCalcModule.getResult() );

			mCalcModule.loadRegisterQ( negative->getCachedPlaneCoeff( plane ) );
			mCalcModule.divideRegisterQ( mCalcModule.getResult() );		                            
		}
		
		inline void init_plane_coeff ( const Ray * const positive,
		                               const Ray * const negative,
		                               const uint_fast16_t plane,
		                               const uint_fast16_t total_planes,
                                       SingleSetRay * const new_ray ) {
	
            prepare_utils( positive, negative, plane );            
            
			for ( uint_fast16_t position = mPlaneCoeffOffset; position < total_planes; position++ ) {
                init_single_coeff( positive, negative, position, new_ray );
			}	
		}        

        protected:
        using PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>::mCacheNode;
        
        void create ( const SingleSetRay * const positive,
                      const SingleSetRay * const negative ) {
                                 
            SingleSetRay * ray = new SingleSetRay( positive, negative, mPlane, GLOBAL_MATRIX_ROWS, mPlaneCoeffOffset );
            init_plane_coeff( positive, negative, mPlane, GLOBAL_MATRIX_ROWS, ray );
                    
            if ( mCacheNode.push( std::shared_ptr<const SingleSetRay>( ray ) ) ) {
                inc_extremal();
            } else {
                inc_verified();
            }
        }

        public:
        SingleSetTraverseVisitor( const TreeNode * const tree,
                                  const RayWrapperContainer<SingleSetRay> & container,
                                  const uint_fast16_t stack_size,
                                  const uint_fast16_t plane,
                                  CacheEntryNode<SingleSetRay, CACHE_ENTRY_TYPE> & node ) :
            PreliminaryTraverseVisitor<SingleSetRay, CACHE_ENTRY_TYPE>( tree, container, stack_size, plane, node ) {}       
    };
}

#endif // __DD_SINGLE_SET_TRAVERSE_VISITOR__
