/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_STATIC_POPULATION_SPLIT_VISITOR__
#define __DD_STATIC_POPULATION_SPLIT_VISITOR__

#include "DDPopulationSplitVisitor.h"

namespace addibit {

    class StaticPopulationSplitVisitor : public PopulationSplitVisitor {

        private:
        unsigned int mMinDistance;

        const std::vector<BitList> mPoints;
        unsigned int mCurrent;

        std::vector<unsigned int> mDistStats;
        std::vector<Interval> mDistPopRange;
        unsigned int mTotal;

        protected:

        void onInit ( const RayReference & ref ) {
            const unsigned int dist = mVantagePoint.getCommonPopCnt( ref.activeSet );

            mDistStats[ dist ]++;
            mDistPopRange[ dist ].update( ref.degeneracy );
            mTotal++;    
        }

        unsigned int getMinDistance () const {
            return mMinDistance;
        }

        bool can_proceed ( const unsigned int index ) const {
            return PopulationSplitVisitor::can_proceed( index ) && /*index > 0 &&*/ mCurrent < mPoints.size();
        }

        public:
        StaticPopulationSplitVisitor ( const unsigned int depth,
                                       const unsigned int capacity, 
                                       const std::vector<BitList> & points,
                                       const unsigned int min_common_bits ) :
            PopulationSplitVisitor( depth, capacity, min_common_bits ),
            mMinDistance( min_common_bits / 2 ),
            mPoints( points ),
            mCurrent( 0 ) {}

        ~StaticPopulationSplitVisitor () {}

        void before ( const RayReferenceContainer & container ) {
            PopulationSplitVisitor::before( container );

            mVantagePoint = mPoints[ mCurrent++ ];
            mDistStats.resize( mVantagePoint.getPopulationCount() + 1, 0 );
            mDistPopRange.resize( mDistStats.size() );
            mTotal = 0;

            mInitRound = true;
        }

        void onStart () {
            PopulationSplitVisitor::onStart();

            if ( !mInitRound ) {
                unsigned int current = 0;

         /*       for ( unsigned int set_index = 0; set_index < mDistStats.size(); set_index++ ) {
                    current += mDistStats[ set_index ];

                    if ( ( current > mTotal * 0.05  ) || set_index == mMinCommonBits - 2 ) {
                        mMinDistance = set_index;
                        break;
                    }
                }                */

                mDistStats.clear();
                mDistPopRange.clear();
            }
        }

        void after () {
            PopulationSplitVisitor::after();
            mCurrent--;
        }     
    };
}

#endif // __DD_STATIC_POPULATION_SPLIT_VISITOR__
