/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DD_STATS__
#define __DD_STATS__

#include <sys/time.h>

#define START_TIMER(ID) struct timeval start_##ID, end_##ID; \
	                    long mtime_##ID, seconds_##ID, useconds_##ID; \
                        gettimeofday( &start_##ID, NULL );

#define STOP_TIMER(ID)  gettimeofday( &end_##ID, NULL ); \
                        seconds_##ID  = end_##ID.tv_sec  - start_##ID.tv_sec; \
	                    useconds_##ID = end_##ID.tv_usec - start_##ID.tv_usec; \
                        mtime_##ID = ((seconds_##ID) * 1000 + useconds_##ID/1000.0) + 0.5;

#define TIMER_VALUE(ID) mtime_##ID

namespace addibit {

    struct StepStats {
#ifdef __STATS_OUTPUT__
        double_t narrowing_ops_linear;
        double_t narrowing_ops_logarithmic;
        double_t narrowing_ops_actual;
        uint_fast64_t narrowing_queries;
        uint_fast64_t nontrivial_candidates;
        uint_fast64_t tree_creation_time;
        
        double_t verification_ops_linear;
        double_t verification_ops_logarithmic;
        double_t verification_ops_actual;
        
        double_t narrowing_complexity;
        double_t verification_complexity;
#endif           
        double_t degeneracy;
        uint_fast64_t narrowed;
        uint_fast64_t verified;
        uint_fast64_t extremal;     

        StepStats () :
#ifdef __STATS_OUTPUT__    
            narrowing_ops_linear( 0.0 ),
            narrowing_ops_logarithmic( 0.0),
            narrowing_ops_actual( 0.0 ),
            narrowing_queries( 0 ),
            nontrivial_candidates( 0 ),
            tree_creation_time( 0 ),
            verification_ops_linear( 0.0 ),
            verification_ops_logarithmic( 0.0),
            verification_ops_actual( 0.0 ),
            narrowing_complexity( 0.0 ),
            verification_complexity( 0.0 ),
#endif             
            degeneracy( 0.0 ),
            narrowed( 0 ),
            verified( 0 ),
            extremal( 0 ) {}
    };
}

#endif // __DD_STATS__
