/*  This is part of Addibit.
    Copyright (C) 2013 Blagoy Genov

    Addibit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __DD_TREE_RAY_CONTAINER__
#define __DD_TREE_RAY_CONTAINER__

#include <vector>
#include "DDBitList.h"

namespace addibit {

	enum RayPosition { POSITIVE, NEGATIVE, ZERO };

	template <class RAY_TYPE>
	class TreeRayContainer {

		private:
		std::vector<const RAY_TYPE*> mPositive;
		std::vector<const RAY_TYPE*> mZero;
		std::vector<const RAY_TYPE*> mNegative;

		BitList mPositiveUnion;
		BitList mNegativeUnion;
		BitList mZeroUnion;
	
		public:
		TreeRayContainer( const unsigned int blocks ) :
			mPositiveUnion( blocks ),
			mNegativeUnion( blocks ),
			mZeroUnion( blocks ) {}

		void insert( const RAY_TYPE* const ray, const RayPosition pos ) {

			switch ( pos ) {
				case POSITIVE:
					insertPositive( ray );
					break;
				case NEGATIVE:
					insertNegative( ray );
					break;		
				case ZERO:
					insertZero( ray );
			}							
		}	

		void insertPositive ( const RAY_TYPE* const ray ) {
			mPositiveUnion |= ray->getActiveSet();
			mPositive.push_back( ray );
		}

		void insertNegative ( const RAY_TYPE* const ray ) {
			mNegativeUnion |= ray->getActiveSet();
			mNegative.push_back( ray );
		}

		void insertZero ( const RAY_TYPE* const ray ) {
			mZeroUnion |= ray->getActiveSet();
			mZero.push_back( ray );	
		}

		const std::vector<const RAY_TYPE*> & getPositiveRays () const {
			return mPositive;
		}

		const BitList & getPositiveUnion () const {
			return mPositiveUnion;
		}

		const std::vector<const RAY_TYPE*> & getNegativeRays () const {
			return mNegative;
		}

		const BitList & getNegativeUnion () const {
			return mNegativeUnion;
		}

		const std::vector<const RAY_TYPE*> & getZeroRays () const {
			return mZero;
		}

		const BitList & getZeroUnion () const {
			return mZeroUnion;
		}

		unsigned long getRaysCount() const {
			return mPositive.size() + mNegative.size() + mZero.size();
		}
	};
}

#endif // __DD_POPULATION_TREE_RAY_CONTAINER__
