
#include "DDBitList.h"

#include <stdlib.h>
#include <fstream>
#include <iostream>

#include <vector>
#include "DDLogging.h"

#define __VERTEX_SET_BLOCKS__ 1

using namespace addibit;

unsigned int load ( std::vector<BitList>& edges, const char* file ) {
	std::ifstream input;
	unsigned int edges_count, vertices_count, first, second;

	input.open( file );	

	if ( !input ) {
		throw "Can not open file!";
	}

	input >> vertices_count;
	input >> edges_count;

	for ( unsigned int i = 0; i < edges_count; i++ ) {
		input >> first;
		input >> second;

		BitList edge( __VERTEX_SET_BLOCKS__ );
		edge.setBit( first );
		edge.setBit( second ); 

		edges.push_back( edge );
	}

	return vertices_count;
}

void generateCuts( const BitList & pattern, const unsigned char start, const unsigned char vertices, std::vector<BitList>& cuts ) {
	
	for ( unsigned int vertex = start; vertex < vertices; vertex++ ) {

		BitList cut( pattern );
		cut.setBit( vertex );

		cuts.push_back( cut );

		if ( ( cut.isSet( 0 ) && vertices % 2 == 0 && cut.getPopulationCount() == ( vertices/2 - 1 )) || ( cut.getPopulationCount() < (vertices-1)/2 ) ) {
			generateCuts( cut, vertex + 1, vertices, cuts );	
		}
	}
}

void generateCuts( const unsigned char vertices, std::vector<BitList>& cuts ) {

	const BitList empty_cut( __VERTEX_SET_BLOCKS__ );

	cuts.push_back( empty_cut );
	generateCuts( empty_cut, 0, vertices, cuts );
}

void generateOutput( const std::vector<BitList>& edges, const std::vector<BitList>& cuts ) {

    std::cout << "begin" << std::endl;
	std::cout << cuts.size() << " " << ( edges.size() + 1 ) << " integer" << std::endl;

	for ( unsigned int cut = 0; cut < cuts.size(); cut++ ) {

		std::cout << 1 << " ";

		for ( unsigned int edge = 0; edge < edges.size(); edge++ ) {

			BitList intersect = cuts[cut] & edges[edge];
			if ( intersect == edges[edge] || intersect.isEmpty() ) {
				std::cout << 0 << " ";
			} else {
				std::cout << 1 << " ";
			}
		}	
		std::cout << std::endl;
	}
    std::cout << "end" << std::endl;
}

void generateOutputForPorta( const std::vector<BitList>& edges, const std::vector<BitList>& cuts ) {

	std::cout << cuts.size() << " " << ( edges.size() + 1 ) << std::endl;

	for ( unsigned int cut = 0; cut < cuts.size(); cut++ ) {

		std::cout << "x1 ";

		for ( unsigned int edge = 0; edge < edges.size(); edge++ ) {

			BitList intersect = cuts[cut] & edges[edge];
			if ( intersect == edges[edge] || intersect.isEmpty() ) {
			//	std::cout << 0 << " ";
			} else {
				std::cout << "+ x" << (edge+2) << " ";
			}
		}	
		std::cout << " >= 0" << std::endl;
	}
}

int main ( int argc, const char** argv ) {

	std::vector<BitList> edges;
	std::vector<BitList> cuts;

	unsigned char vertices = load( edges, argv[1] );

	generateCuts( vertices, cuts );
	generateOutput( edges, cuts );

	return 0;
}
